from pathlib import Path
from tempfile import TemporaryDirectory
import unittest

from skilgen.core.config import load_config, render_default_config


class ConfigTests(unittest.TestCase):
    def test_render_default_config_is_provider_neutral(self) -> None:
        rendered = render_default_config()
        self.assertIn("model_provider:", rendered)
        self.assertNotIn("model_provider: openai", rendered)
        self.assertIn("# openai / gpt-4.1-mini / OPENAI_API_KEY", rendered)
        self.assertIn("update_trigger: auto", rendered)

    def test_render_default_config_can_scaffold_provider_defaults(self) -> None:
        rendered = render_default_config("gemini")
        self.assertIn("model_provider: gemini", rendered)
        self.assertIn("model: gemini-2.5-pro", rendered)
        self.assertIn("api_key_env: GOOGLE_API_KEY", rendered)

    def test_load_config_reads_lists_and_scalars(self) -> None:
        with TemporaryDirectory() as tmp:
            root = Path(tmp)
            (root / "skilgen.yml").write_text(
                "\n".join(
                    [
                        "include_paths:",
                        "  - src",
                        "exclude_paths:",
                        "  - .git",
                        "domains_override:",
                        "  - backend",
                        "skill_depth: 3",
                        "update_trigger: watch",
                        "langsmith_project: skilgen-dev",
                        "model_provider: gemini",
                        "model: gpt-5",
                        "api_key_env: CUSTOM_OPENAI_KEY",
                        "model_temperature: 0.2",
                        "model_max_tokens: 2048",
                        "model_retry_attempts: 5",
                        "model_retry_base_delay_seconds: 2.5",
                        "external_skills_policy_mode: review_required",
                    ]
                ),
                encoding="utf-8",
            )
            config = load_config(root)
            self.assertEqual(config.include_paths, ["src"])
            self.assertEqual(config.exclude_paths, [".git"])
            self.assertEqual(config.domains_override, ["backend"])
            self.assertEqual(config.skill_depth, 3)
            self.assertEqual(config.update_trigger, "watch")
            self.assertEqual(config.langsmith_project, "skilgen-dev")
            self.assertEqual(config.model_provider, "gemini")
            self.assertEqual(config.model, "gpt-5")
            self.assertEqual(config.api_key_env, "CUSTOM_OPENAI_KEY")
            self.assertEqual(config.model_temperature, 0.2)
            self.assertEqual(config.model_max_tokens, 2048)
            self.assertEqual(config.model_retry_attempts, 5)
            self.assertEqual(config.model_retry_base_delay_seconds, 2.5)
            self.assertEqual(config.external_skills_policy_mode, "review_required")


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import argparse
import json
import sys
from pathlib import Path

from skilgen.api.server import run_server
from skilgen.autoupdate import auto_update_status, ensure_auto_update_worker, run_auto_update_worker, stop_auto_update_worker
from skilgen.api.service import analyze_payload, decision_payload, doctor_payload, preview_payload, report_payload, status_payload, validate_payload
from skilgen import __version__
from skilgen.agents import build_import_graph, build_roadmap_plan, extract_features, fingerprint_project
from skilgen.agents.requirements_parser import parse_project_intent, parse_requirements_file
from skilgen.deep_agents_core import current_runtime_mode, runtime_diagnostics
from skilgen.delivery import run_delivery, watch_delivery
from skilgen.core.config import load_config, render_default_config
from skilgen.enterprise_skills import (
    activate_mcp_connector,
    active_mcp_connectors,
    connector_catalog,
    deactivate_mcp_connector,
    generate_enterprise_skill,
    ingest_enterprise_skill,
    list_enterprise_skills,
    recommend_mcp_connectors,
)
from skilgen.external_skills import (
    activate_external_skill,
    active_external_skills,
    detect_external_skill_sources,
    external_skill_lock,
    external_skill_policy,
    deactivate_external_skill,
    export_external_skill_lock,
    get_external_skill,
    import_external_skill_candidates,
    import_external_skill_lock,
    install_external_skill,
    list_external_skills,
    ranked_external_skills,
    remove_external_skill,
    sync_all_external_skills,
    sync_external_skill,
)


def emit_progress(message: str) -> None:
    print(f"[skilgen] {message}", file=sys.stderr)


def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(prog="skilgen", description="Requirements-driven skill and scaffold generator.")
    parser.add_argument("--version", action="version", version=f"%(prog)s {__version__}")
    subparsers = parser.add_subparsers(dest="command", required=True)

    init = subparsers.add_parser("init", help="Write a default skilgen.yml to the project root.")
    init.add_argument("--project-root", default=".")
    init.add_argument(
        "--provider",
        choices=["openai", "anthropic", "gemini", "google", "google_genai", "huggingface", "hugging_face", "hf"],
        help="Optionally scaffold provider-specific model defaults instead of a neutral template.",
    )

    scan = subparsers.add_parser("scan", help="Generate docs and skills from a requirements file.")
    scan.add_argument("--requirements")
    scan.add_argument("--project-root", default=".")
    scan.add_argument("--target", choices=["all", "docs", "skills"], default="all")
    scan.add_argument("--domain", action="append", choices=["requirements", "backend", "frontend", "roadmap"])
    scan.add_argument("--dry-run", action="store_true")

    deliver = subparsers.add_parser("deliver", help="Alias for scan for now; intended to grow into full delivery automation.")
    deliver.add_argument("--requirements")
    deliver.add_argument("--project-root", default=".")
    deliver.add_argument("--target", choices=["all", "docs", "skills"], default="all")
    deliver.add_argument("--domain", action="append", choices=["requirements", "backend", "frontend", "roadmap"])
    deliver.add_argument("--dry-run", action="store_true")

    update = subparsers.add_parser("update", help="Refresh generated outputs for all or selected domains.")
    update.add_argument("--requirements")
    update.add_argument("--project-root", default=".")
    update.add_argument("--target", choices=["all", "docs", "skills"], default="all")
    update.add_argument("--domain", action="append", choices=["requirements", "backend", "frontend", "roadmap"])
    update.add_argument("--dry-run", action="store_true")

    watch = subparsers.add_parser("watch", help="Watch the project and rerun generation when files change.")
    watch.add_argument("--requirements")
    watch.add_argument("--project-root", default=".")
    watch.add_argument("--target", choices=["all", "docs", "skills"], default="all")
    watch.add_argument("--domain", action="append", choices=["requirements", "backend", "frontend", "roadmap"])
    watch.add_argument("--interval", type=float, default=2.0)
    watch.add_argument("--cycles", type=int, default=0)
    watch.add_argument("--once", action="store_true")

    autoupdate = subparsers.add_parser("autoupdate", help="Manage Skilgen's background skill auto-update worker.")
    autoupdate_subparsers = autoupdate.add_subparsers(dest="autoupdate_command", required=True)

    autoupdate_enable = autoupdate_subparsers.add_parser("enable", help="Start the repo-local auto-update worker.")
    autoupdate_enable.add_argument("--project-root", default=".")
    autoupdate_enable.add_argument("--requirements")
    autoupdate_enable.add_argument("--interval", type=float, default=2.0)

    autoupdate_status_parser = autoupdate_subparsers.add_parser("status", help="Show the current auto-update worker status.")
    autoupdate_status_parser.add_argument("--project-root", default=".")

    autoupdate_disable = autoupdate_subparsers.add_parser("disable", help="Stop the repo-local auto-update worker.")
    autoupdate_disable.add_argument("--project-root", default=".")

    autoupdate_worker = autoupdate_subparsers.add_parser("worker", help=argparse.SUPPRESS)
    autoupdate_worker.add_argument("--project-root", default=".")
    autoupdate_worker.add_argument("--interval", type=float, default=2.0)

    preview = subparsers.add_parser("preview", help="Preview which generated files would be written without changing the project.")
    preview.add_argument("--requirements")
    preview.add_argument("--project-root", default=".")
    preview.add_argument("--target", choices=["all", "docs", "skills"], default="all")
    preview.add_argument("--domain", action="append", choices=["requirements", "backend", "frontend", "roadmap"])

    fingerprint = subparsers.add_parser("fingerprint", help="Detect the current project's likely frameworks.")
    fingerprint.add_argument("--project-root", default=".")

    mapping = subparsers.add_parser("map", help="Build a simple import relationship map for the project.")
    mapping.add_argument("--project-root", default=".")

    analyze = subparsers.add_parser("analyze", help="Assemble framework, signal, and relationship analysis for the project.")
    analyze.add_argument("--project-root", default=".")
    analyze.add_argument("--requirements")

    decide = subparsers.add_parser("decide", help="Recommend whether to refresh skills, which skills to prioritize, and which run memory to load.")
    decide.add_argument("--project-root", default=".")
    decide.add_argument("--requirements")

    skills = subparsers.add_parser("skills", help="Discover and install external skill collections through Skilgen.")
    skills_subparsers = skills.add_subparsers(dest="skills_command", required=True)

    skills_list = skills_subparsers.add_parser("list", help="List curated external skill sources available through Skilgen.")
    skills_list.add_argument("--project-root", default=".")
    skills_list.add_argument("--ecosystem")
    skills_list.add_argument("--search")

    skills_show = skills_subparsers.add_parser("show", help="Show details for a curated external skill source.")
    skills_show.add_argument("slug")
    skills_show.add_argument("--project-root", default=".")

    skills_detect = skills_subparsers.add_parser("detect", help="Detect external skill ecosystems that match the current repository.")
    skills_detect.add_argument("--project-root", default=".")

    skills_active = skills_subparsers.add_parser("active", help="List the currently active external skill packs for this project.")
    skills_active.add_argument("--project-root", default=".")

    skills_lock = skills_subparsers.add_parser("lock", help="Show the resolved external-skills lockfile for this project.")
    skills_lock.add_argument("--project-root", default=".")

    skills_lock_export = skills_subparsers.add_parser("lock-export", help="Export the resolved external-skills lockfile for reuse in another repo.")
    skills_lock_export.add_argument("--project-root", default=".")
    skills_lock_export.add_argument("--output-path")

    skills_lock_import = skills_subparsers.add_parser("lock-import", help="Import an exported external-skills lockfile into this project.")
    skills_lock_import.add_argument("--project-root", default=".")
    skills_lock_import.add_argument("--input-path", required=True)
    skills_lock_import.add_argument("--sync-existing", action="store_true")

    skills_policy = skills_subparsers.add_parser("policy", help="Show the external-skills policy currently applied to this project.")
    skills_policy.add_argument("--project-root", default=".")

    skills_rank = skills_subparsers.add_parser("rank", help="Rank active external skill packs by trust and relevance for the current project.")
    skills_rank.add_argument("--project-root", default=".")

    skills_install = skills_subparsers.add_parser("install", help="Install a curated or custom external skill source into the local project.")
    skills_install.add_argument("slug", nargs="?")
    skills_install.add_argument("--git-url")
    skills_install.add_argument("--name")
    skills_install.add_argument("--project-root", default=".")
    skills_install.add_argument("--force", action="store_true")
    skills_install.add_argument("--ref")
    skills_install.add_argument("--activate", action=argparse.BooleanOptionalAction, default=None)

    skills_import = skills_subparsers.add_parser("import", help="Import downstream repos from an installed directory-style external skill source.")
    skills_import.add_argument("slug")
    skills_import.add_argument("--project-root", default=".")
    skills_import.add_argument("--limit", type=int, default=5)
    skills_import.add_argument("--activate", action=argparse.BooleanOptionalAction, default=None)

    skills_sync = skills_subparsers.add_parser("sync", help="Sync an installed external skill source with its upstream repository.")
    skills_sync.add_argument("slug", nargs="?")
    skills_sync.add_argument("--project-root", default=".")
    skills_sync.add_argument("--all", action="store_true")

    skills_remove = skills_subparsers.add_parser("remove", help="Remove an installed external skill source from the local project.")
    skills_remove.add_argument("slug")
    skills_remove.add_argument("--project-root", default=".")

    skills_activate = skills_subparsers.add_parser("activate", help="Mark an installed external skill source as active for agent loading.")
    skills_activate.add_argument("slug")
    skills_activate.add_argument("--project-root", default=".")

    skills_deactivate = skills_subparsers.add_parser("deactivate", help="Mark an installed external skill source as inactive for agent loading.")
    skills_deactivate.add_argument("slug")
    skills_deactivate.add_argument("--project-root", default=".")

    enterprise = subparsers.add_parser("enterprise", help="Ingest and generate enterprise-wide skill packs for coding agents.")
    enterprise_subparsers = enterprise.add_subparsers(dest="enterprise_command", required=True)

    enterprise_list = enterprise_subparsers.add_parser("list", help="List enterprise skill packs installed for this project.")
    enterprise_list.add_argument("--project-root", default=".")

    enterprise_ingest = enterprise_subparsers.add_parser("ingest", help="Ingest an existing enterprise skill pack from a local path or git repo.")
    enterprise_ingest.add_argument("--project-root", default=".")
    enterprise_ingest.add_argument("--name", required=True)
    enterprise_ingest.add_argument("--path")
    enterprise_ingest.add_argument("--git-url")
    enterprise_ingest.add_argument("--ref")
    enterprise_ingest.add_argument("--kind", default="enterprise")
    enterprise_ingest.add_argument("--activate", action=argparse.BooleanOptionalAction, default=None)

    enterprise_generate = enterprise_subparsers.add_parser("generate", help="Generate an enterprise skill from docs, code, or runbooks.")
    enterprise_generate.add_argument("--project-root", default=".")
    enterprise_generate.add_argument("--name", required=True)
    enterprise_generate.add_argument("--kind", default="domain")
    enterprise_generate.add_argument("--source-path", action="append", required=True)
    enterprise_generate.add_argument("--activate", action=argparse.BooleanOptionalAction, default=True)

    connectors = subparsers.add_parser("connectors", help="Discover and manage approved MCP connector capabilities.")
    connectors_subparsers = connectors.add_subparsers(dest="connectors_command", required=True)

    connectors_list = connectors_subparsers.add_parser("list", help="List supported MCP connectors.")
    connectors_list.add_argument("--system")
    connectors_list.add_argument("--search")

    connectors_recommend = connectors_subparsers.add_parser("recommend", help="Recommend MCP connectors for the current project.")
    connectors_recommend.add_argument("--project-root", default=".")

    connectors_active = connectors_subparsers.add_parser("active", help="List active MCP connectors for the current project.")
    connectors_active.add_argument("--project-root", default=".")

    connectors_activate = connectors_subparsers.add_parser("activate", help="Activate an MCP connector for this project.")
    connectors_activate.add_argument("slug")
    connectors_activate.add_argument("--project-root", default=".")

    connectors_deactivate = connectors_subparsers.add_parser("deactivate", help="Deactivate an MCP connector for this project.")
    connectors_deactivate.add_argument("slug")
    connectors_deactivate.add_argument("--project-root", default=".")

    intent = subparsers.add_parser("intent", help="Parse a requirements file into a structured project intent.")
    intent.add_argument("--requirements", required=True)
    features = subparsers.add_parser("features", help="Extract a feature inventory from requirements and project context.")
    features.add_argument("--requirements")
    features.add_argument("--project-root", default=".")
    plan = subparsers.add_parser("plan", help="Build a roadmap plan from requirements and model config.")
    plan.add_argument("--requirements")
    plan.add_argument("--project-root", default=".")

    status = subparsers.add_parser("status", help="Show the current generated output status for a project root.")
    status.add_argument("--project-root", default=".")

    report = subparsers.add_parser("report", help="Show a summary report for a project root.")
    report.add_argument("--project-root", default=".")

    validate = subparsers.add_parser("validate", help="Validate generated outputs and skill references.")
    validate.add_argument("--project-root", default=".")

    doctor = subparsers.add_parser("doctor", help="Diagnose runtime readiness, model configuration, and API-key setup.")
    doctor.add_argument("--project-root", default=".")

    serve = subparsers.add_parser("serve", help="Run the HTTP API server.")
    serve.add_argument("--host", default="127.0.0.1")
    serve.add_argument("--port", type=int, default=8000)
    return parser


def main() -> None:
    parser = build_parser()
    args = parser.parse_args()
    if args.command == "init":
        project_root = Path(args.project_root).resolve()
        project_root.mkdir(parents=True, exist_ok=True)
        config_path = project_root / "skilgen.yml"
        if not config_path.exists():
            config_path.write_text(render_default_config(args.provider), encoding="utf-8")
        worker = ensure_auto_update_worker(project_root)
        print(json.dumps({"config_path": str(config_path), "auto_update": worker}, indent=2))
        return
    if args.command == "autoupdate":
        root = Path(args.project_root).resolve()
        if args.autoupdate_command == "enable":
            payload = ensure_auto_update_worker(
                root,
                requirements_path=Path(args.requirements).resolve() if args.requirements else None,
                interval_seconds=args.interval,
            )
            print(json.dumps(payload, indent=2))
            return
        if args.autoupdate_command == "status":
            print(json.dumps(auto_update_status(root), indent=2))
            return
        if args.autoupdate_command == "disable":
            print(json.dumps(stop_auto_update_worker(root), indent=2))
            return
        if args.autoupdate_command == "worker":
            run_auto_update_worker(root, interval_seconds=args.interval)
            return
    if args.command == "fingerprint":
        result = fingerprint_project(Path(args.project_root).resolve())
        print(
            json.dumps(
                {
                    "frontend": result.frontend.__dict__ if result.frontend else None,
                    "backend": result.backend.__dict__ if result.backend else None,
                    "test_framework": result.test_framework.__dict__ if result.test_framework else None,
                    "build_tool": result.build_tool.__dict__ if result.build_tool else None,
                },
                indent=2,
            )
        )
        return
    if args.command == "map":
        print(json.dumps({"import_graph": build_import_graph(Path(args.project_root).resolve())}, indent=2))
        return
    if args.command == "analyze":
        print(json.dumps(analyze_payload(Path(args.project_root).resolve(), Path(args.requirements).resolve() if args.requirements else None), indent=2))
        return
    if args.command == "decide":
        root = Path(args.project_root).resolve()
        emit_progress(
            f"Starting agent decision planning with the {current_runtime_mode(root)} runtime. Skilgen is deciding whether the skill tree should refresh and what the agent should load first."
        )
        print(json.dumps(decision_payload(root, Path(args.requirements).resolve() if args.requirements else None), indent=2))
        return
    if args.command == "skills":
        root = Path(args.project_root).resolve()
        if args.skills_command == "list":
            emit_progress("Loading the curated Skilgen skills catalog across supported ecosystems.")
            print(json.dumps(list_external_skills(root, ecosystem=args.ecosystem, search=args.search), indent=2))
            return
        if args.skills_command == "show":
            emit_progress(f"Loading details for the external skill source '{args.slug}'.")
            print(json.dumps({"skill": get_external_skill(args.slug, root)}, indent=2))
            return
        if args.skills_command == "detect":
            emit_progress("Scanning the repository for supported external skill ecosystems.")
            print(json.dumps(detect_external_skill_sources(root), indent=2))
            return
        if args.skills_command == "active":
            emit_progress("Listing the currently active external skill packs for this project.")
            print(json.dumps({"skills": active_external_skills(root)}, indent=2))
            return
        if args.skills_command == "lock":
            emit_progress("Loading the resolved external-skills lockfile.")
            print(json.dumps(external_skill_lock(root), indent=2))
            return
        if args.skills_command == "lock-export":
            emit_progress("Exporting the resolved external-skills lockfile for reuse in another project.")
            print(json.dumps(export_external_skill_lock(project_root=root, output_path=args.output_path), indent=2))
            return
        if args.skills_command == "lock-import":
            emit_progress("Importing external skills from an exported lockfile into this project.")
            print(json.dumps(import_external_skill_lock(project_root=root, input_path=args.input_path, sync_existing=args.sync_existing), indent=2))
            return
        if args.skills_command == "policy":
            emit_progress("Loading the external-skills policy for this project.")
            print(json.dumps(external_skill_policy(root), indent=2))
            return
        if args.skills_command == "rank":
            emit_progress("Ranking active external skill packs by trust, detection signals, and repo fit.")
            print(json.dumps(ranked_external_skills(root), indent=2))
            return
        if args.skills_command == "install":
            emit_progress("Installing the external skill source into .skilgen/external-skills so it can be managed through Skilgen.")
            print(
                json.dumps(
                    {
                        "installed_skill": install_external_skill(
                            project_root=root,
                            slug=args.slug,
                            git_url=args.git_url,
                            name=args.name,
                            force=args.force,
                            ref=args.ref,
                            active=args.activate,
                        )
                    },
                    indent=2,
                )
            )
            return
        if args.skills_command == "import":
            emit_progress("Importing downstream repositories from the selected directory-style skill source.")
            print(json.dumps(import_external_skill_candidates(project_root=root, slug=args.slug, limit=args.limit, active=args.activate), indent=2))
            return
        if args.skills_command == "sync":
            if args.all:
                emit_progress("Syncing all installed external skill sources with their upstream repositories.")
                print(json.dumps(sync_all_external_skills(project_root=root), indent=2))
            else:
                emit_progress(f"Syncing the external skill source '{args.slug}' with its upstream repository.")
                print(json.dumps({"synced_skill": sync_external_skill(project_root=root, slug=args.slug)}, indent=2))
            return
        if args.skills_command == "remove":
            emit_progress(f"Removing the external skill source '{args.slug}' from the local Skilgen registry.")
            print(json.dumps({"removed_skill": remove_external_skill(project_root=root, slug=args.slug)}, indent=2))
            return
        if args.skills_command == "activate":
            emit_progress(f"Activating the external skill source '{args.slug}' for agent loading.")
            print(json.dumps({"activated_skill": activate_external_skill(project_root=root, slug=args.slug)}, indent=2))
            return
        if args.skills_command == "deactivate":
            emit_progress(f"Deactivating the external skill source '{args.slug}' for agent loading.")
            print(json.dumps({"deactivated_skill": deactivate_external_skill(project_root=root, slug=args.slug)}, indent=2))
            return
    if args.command == "enterprise":
        root = Path(args.project_root).resolve()
        if args.enterprise_command == "list":
            emit_progress("Loading enterprise skill packs installed for this project.")
            print(json.dumps({"skills": list_enterprise_skills(root)}, indent=2))
            return
        if args.enterprise_command == "ingest":
            emit_progress("Ingesting an existing enterprise skill pack into .skilgen/enterprise-skills.")
            print(
                json.dumps(
                    {
                        "enterprise_skill": ingest_enterprise_skill(
                            root,
                            name=args.name,
                            path=args.path,
                            git_url=args.git_url,
                            ref=args.ref,
                            activate=args.activate,
                            kind=args.kind,
                        )
                    },
                    indent=2,
                )
            )
            return
        if args.enterprise_command == "generate":
            emit_progress("Generating a reusable enterprise skill from the provided docs, code, or runbooks.")
            print(
                json.dumps(
                    {
                        "enterprise_skill": generate_enterprise_skill(
                            root,
                            name=args.name,
                            source_paths=args.source_path,
                            kind=args.kind,
                            activate=args.activate,
                        )
                    },
                    indent=2,
                )
            )
            return
    if args.command == "connectors":
        root = Path(args.project_root).resolve() if hasattr(args, "project_root") else Path(".").resolve()
        if args.connectors_command == "list":
            emit_progress("Loading supported MCP connectors for enterprise coding workflows.")
            print(json.dumps(connector_catalog(system=args.system, search=args.search), indent=2))
            return
        if args.connectors_command == "recommend":
            emit_progress("Scanning the repo for enterprise systems and recommending useful MCP connectors.")
            print(json.dumps(recommend_mcp_connectors(root), indent=2))
            return
        if args.connectors_command == "active":
            emit_progress("Listing active MCP connectors configured for this project.")
            print(json.dumps({"connectors": active_mcp_connectors(root)}, indent=2))
            return
        if args.connectors_command == "activate":
            emit_progress(f"Activating the MCP connector '{args.slug}' for this project.")
            print(json.dumps({"connector": activate_mcp_connector(root, args.slug)}, indent=2))
            return
        if args.connectors_command == "deactivate":
            emit_progress(f"Deactivating the MCP connector '{args.slug}' for this project.")
            print(json.dumps({"connector": deactivate_mcp_connector(root, args.slug)}, indent=2))
            return
    if args.command == "intent":
        result = parse_requirements_file(Path(args.requirements).resolve())
        print(
            json.dumps(
                {
                    "features": result.features,
                    "domain_concepts": result.domain_concepts,
                    "entities": result.entities,
                    "endpoints": result.endpoints,
                    "ui_flows": result.ui_flows,
                },
                indent=2,
            )
        )
        return
    if args.command == "features":
        root = Path(args.project_root).resolve()
        emit_progress(
            f"Starting feature synthesis with the {current_runtime_mode(root)} runtime. Skilgen is reading the project context to identify the capabilities that matter."
        )
        emit_progress("Reading the codebase and optional requirements to identify product capabilities.")
        requirements = Path(args.requirements).resolve() if args.requirements else None
        features = extract_features(requirements, root)
        emit_progress("Grouping detected backend, frontend, and planning signals into a reusable feature inventory.")
        print(json.dumps({"features": [feature.__dict__ for feature in features]}, indent=2))
        return
    if args.command == "plan":
        root = Path(args.project_root).resolve()
        emit_progress(
            f"Starting roadmap planning with the {current_runtime_mode(root)} runtime. Skilgen is turning project context into a staged implementation plan."
        )
        emit_progress("Reading project scope and available inputs for roadmap planning.")
        config = load_config(root)
        plan = build_roadmap_plan(
            config,
            parse_project_intent(root, Path(args.requirements).resolve() if args.requirements else None),
            root,
        )
        emit_progress("Synthesizing implementation phases and sequencing the next delivery steps.")
        print(
            json.dumps(
                {
                    "model": plan.model.__dict__,
                    "steps": [step.__dict__ for step in plan.steps],
                },
                indent=2,
            )
        )
        return
    if args.command == "status":
        print(json.dumps(status_payload(Path(args.project_root).resolve()), indent=2))
        return
    if args.command == "report":
        print(json.dumps(report_payload(Path(args.project_root).resolve()), indent=2))
        return
    if args.command == "doctor":
        payload = doctor_payload(Path(args.project_root).resolve())
        print(json.dumps(payload, indent=2))
        return
    if args.command == "validate":
        print(json.dumps(validate_payload(Path(args.project_root).resolve()), indent=2))
        return
    if args.command == "serve":
        run_server(args.host, args.port)
        return

    targets = ("docs", "skills") if getattr(args, "target", "all") == "all" else (args.target,)
    domains = tuple(getattr(args, "domain", None) or [])

    if args.command == "preview":
        print(
            json.dumps(
                preview_payload(
                    Path(args.requirements).resolve() if args.requirements else None,
                    Path(args.project_root),
                    targets=targets,
                    domains=domains,
                ),
                indent=2,
            )
        )
        return

    if args.command == "watch":
        root = Path(args.project_root).resolve()
        emit_progress(
            f"Starting watch mode with the {current_runtime_mode(root)} runtime. Skilgen will explain each refresh as changes are detected."
        )
        runs = watch_delivery(
            Path(args.requirements).resolve() if args.requirements else None,
            root,
            targets=targets,
            domains=domains,
            interval_seconds=args.interval,
            cycles=args.cycles,
            once=args.once,
            progress_callback=emit_progress,
        )
        print(json.dumps({"runtime": current_runtime_mode(root), "runs": [[str(path) for path in generated] for generated in runs]}, indent=2))
        return

    root = Path(args.project_root).resolve()
    ensure_auto_update_worker(root, requirements_path=Path(args.requirements).resolve() if args.requirements else None)
    diagnostics = runtime_diagnostics(root)
    emit_progress(
        f"Starting delivery with the {current_runtime_mode(root)} runtime. This may take a bit while Skilgen builds project context and generates the final skill tree."
    )
    if diagnostics["runtime"] != "model_backed":
        emit_progress(f"Model-backed runtime is not ready: {diagnostics['reason']}")
    generated = run_delivery(
        Path(args.requirements).resolve() if args.requirements else None,
        root,
        targets=targets,
        domains=domains,
        dry_run=args.dry_run,
        progress_callback=emit_progress,
    )
    print(
        json.dumps(
            {
                "runtime": current_runtime_mode(root),
                "runtime_diagnostics": diagnostics,
                "generated_files": [str(path) for path in generated],
            },
            indent=2,
        )
    )


def console_main() -> None:
    main()


if __name__ == "__main__":
    console_main()



from pathlib import Path
from tempfile import TemporaryDirectory
import unittest

from skilgen.agents.requirements_parser import parse_requirements_file


class RequirementsParserTests(unittest.TestCase):
    def test_parse_requirements_file_extracts_project_intent(self) -> None:
        with TemporaryDirectory() as tmp:
            path = Path(tmp) / "requirements.md"
            path.write_text(
                "\n".join(
                    [
                        "Feature: Auto-update engine",
                        "Backend API endpoint for skills scan",
                        "Frontend dashboard flow for skill tracking",
                        "Entity: SkillVersion",
                        "Domain: backend",
                    ]
                ),
                encoding="utf-8",
            )
            intent = parse_requirements_file(path)
            self.assertTrue(intent.features)
            self.assertTrue(intent.endpoints)
            self.assertTrue(intent.ui_flows)
            self.assertTrue(intent.entities)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from pathlib import Path

from skilgen.deep_agents_core import run_deep_json
from skilgen.core.freshness import compute_freshness_report, load_freshness_state
from skilgen.core.models import AgentDecision, RequirementsContext, RunMemory
from skilgen.core.run_memory import load_current_run_memory
from skilgen.enterprise_skills import active_enterprise_skills, active_mcp_connectors
from skilgen.external_skills import external_skill_policy, ranked_external_skills


def build_agent_decision_native(
    project_root: Path,
    requirements: RequirementsContext,
    domain_graph,
    skill_tree,
    current_run_memory: RunMemory | None,
) -> AgentDecision:
    freshness = compute_freshness_report(project_root, requirements, domain_graph, load_freshness_state(project_root))
    top_level_skill_paths = [
        node.path
        for node in skill_tree
        if node.parent_skill is None and (not freshness.impacted_domains or node.domain in freshness.impacted_domains)
    ]
    if not top_level_skill_paths:
        top_level_skill_paths = [node.path for node in skill_tree if node.parent_skill is None]
    ranked_external = ranked_external_skills(project_root).get("skills", [])
    enterprise_skills = active_enterprise_skills(project_root)
    connectors = active_mcp_connectors(project_root)
    memory_to_load = [".skilgen/memory/current_run.json", ".skilgen/state/freshness.json", ".skilgen/external-skills/lock.json"]
    preferred_external: list[str] = []
    if current_run_memory is not None:
        memory_to_load.append(f".skilgen/memory/runs/{current_run_memory.run_id}.json")
    for entry in ranked_external[:3]:
        preferred_external.append(str(entry.get("slug", "")))
        normalized = entry.get("lock_metadata", {}).get("normalized", {}) if isinstance(entry.get("lock_metadata"), dict) else {}
        summary_path = normalized.get("summary_path")
        index_path = normalized.get("index_path")
        if isinstance(summary_path, str):
            try:
                relative = Path(summary_path).resolve().relative_to(project_root).as_posix()
            except ValueError:
                relative = summary_path
            memory_to_load.append(relative)
            if relative not in top_level_skill_paths:
                top_level_skill_paths.append(relative)
        if isinstance(index_path, str):
            try:
                relative_index = Path(index_path).resolve().relative_to(project_root).as_posix()
            except ValueError:
                relative_index = index_path
            memory_to_load.append(relative_index)
    for entry in enterprise_skills[:5]:
        skill_path = entry.get("skill_path") or entry.get("install_path")
        summary_path = entry.get("summary_path")
        for candidate in (skill_path, summary_path):
            if not isinstance(candidate, str):
                continue
            try:
                relative = Path(candidate).resolve().relative_to(project_root).as_posix()
            except ValueError:
                relative = candidate
            if relative not in top_level_skill_paths:
                top_level_skill_paths.append(relative)
            if relative not in memory_to_load:
                memory_to_load.append(relative)
    should_refresh = freshness.reason != "no_source_changes"
    reason = (
        "Source changes were detected and the impacted domains should be refreshed before the next coding task."
        if should_refresh
        else "No source changes were detected, so agents can reuse the current skill tree and run memory."
    )
    next_actions = []
    if should_refresh:
        next_actions.append("Refresh the impacted top-level skills before starting implementation work.")
    next_actions.append("Load AGENTS.md and the prioritized parent skills first.")
    if ranked_external:
        next_actions.append(
            f"Load the top ranked external skill packs first: {', '.join(slug for slug in preferred_external if slug) or 'external summaries'}."
        )
        next_actions.append("Use each external pack's normalized index and summary before loading lower-ranked imports.")
    if enterprise_skills:
        next_actions.append(
            f"Load active enterprise skills next: {', '.join(str(entry.get('slug')) for entry in enterprise_skills[:5])}."
        )
    if connectors:
        next_actions.append(
            f"Use approved MCP connectors only when needed: {', '.join(str(entry.get('slug')) for entry in connectors[:5])}."
        )
    if current_run_memory is not None and current_run_memory.recent_events:
        next_actions.append("Use the latest run memory to continue from the most recent execution context.")
    return AgentDecision(
        should_refresh=should_refresh,
        reason=reason,
        prioritized_domains=freshness.impacted_domains or freshness.top_level_domains,
        prioritized_skill_paths=top_level_skill_paths,
        memory_to_load=memory_to_load,
        next_actions=next_actions,
    )


def build_agent_decision(
    project_root: Path,
    requirements: RequirementsContext,
    domain_graph,
    skill_tree,
) -> AgentDecision:
    root = project_root.resolve()
    current_run_memory = load_current_run_memory(root)
    native = build_agent_decision_native(root, requirements, domain_graph, skill_tree, current_run_memory)
    freshness = compute_freshness_report(root, requirements, domain_graph, load_freshness_state(root))
    policy = external_skill_policy(root)
    ranked = ranked_external_skills(root)
    enterprise = active_enterprise_skills(root)
    connectors = active_mcp_connectors(root)
    payload = run_deep_json(
        "agent decision planning",
        (
            "Decide whether Skilgen should refresh skills now, which domains and skills an agent should prioritize, "
            "and which memory files an agent should load first. Return JSON with keys should_refresh, reason, "
            "prioritized_domains, prioritized_skill_paths, memory_to_load, next_actions. Optimize for agent usefulness: "
            "favor the smallest refresh that preserves correctness, choose skills that match the currently impacted "
            "domains, rank trusted external skill packs when they fit the repo, and keep memory recommendations focused on the files most likely to help the next coding step.\n\n"
            f"Project root: {root}\n"
            f"Freshness JSON: {freshness.__dict__}\n"
            f"External skills policy JSON: {policy}\n"
            f"Ranked external skills JSON: {ranked}\n"
            f"Active enterprise skills JSON: {enterprise}\n"
            f"Active MCP connectors JSON: {connectors}\n"
            f"Current run memory: {current_run_memory.__dict__ if current_run_memory is not None else None}\n"
            f"Skill tree JSON: {[node.__dict__ for node in skill_tree]}\n"
            f"Native decision JSON: {native.__dict__}\n"
        ),
        lambda: native.__dict__,
        project_root=root,
    )
    return AgentDecision(
        should_refresh=bool(payload.get("should_refresh", native.should_refresh)),
        reason=str(payload.get("reason", native.reason)),
        prioritized_domains=[str(item) for item in payload.get("prioritized_domains", native.prioritized_domains)],
        prioritized_skill_paths=[str(item) for item in payload.get("prioritized_skill_paths", native.prioritized_skill_paths)],
        memory_to_load=[str(item) for item in payload.get("memory_to_load", native.memory_to_load)],
        next_actions=[str(item) for item in payload.get("next_actions", native.next_actions)],
    )



from skilgen.agents.codebase_signals import analyze_codebase
from skilgen.agents.decision_planner import build_agent_decision
from skilgen.agents.domain_graph_planner import build_domain_graph
from skilgen.agents.feature_extractor import extract_features
from skilgen.agents.framework_fingerprint import fingerprint_project
from skilgen.agents.model_registry import resolve_model_settings
from skilgen.agents.relationship_mapper import build_import_graph
from skilgen.agents.requirements_parser import parse_requirements_file
from skilgen.agents.roadmap_planner import build_roadmap_plan

__all__ = [
    "analyze_codebase",
    "build_agent_decision",
    "build_domain_graph",
    "build_import_graph",
    "extract_features",
    "fingerprint_project",
    "parse_requirements_file",
    "resolve_model_settings",
    "build_roadmap_plan",
]



from __future__ import annotations

from pathlib import Path
from typing import Callable

from skilgen.api.jobs import get_job, job_payload, list_jobs, request_cancel, submit_job
from skilgen.agents.decision_planner import build_agent_decision
from skilgen.autoupdate import auto_update_status
from skilgen.deep_agents_core import current_runtime_mode, runtime_diagnostics
from skilgen.deep_agents_runtime import (
    DeepAgentsRuntime,
    native_analyze_payload,
    native_features_payload,
    native_fingerprint_payload,
    native_intent_payload,
    native_map_payload,
    native_plan_payload,
    native_preview_payload,
    native_report_payload,
    native_status_payload,
    native_validate_payload,
)
from skilgen.delivery import run_delivery
from skilgen.enterprise_skills import (
    activate_mcp_connector,
    active_enterprise_skills,
    active_mcp_connectors,
    connector_catalog,
    deactivate_mcp_connector,
    generate_enterprise_skill,
    ingest_enterprise_skill,
    list_enterprise_skills,
    recommend_mcp_connectors,
)
from skilgen.core.freshness import compute_freshness_report, load_freshness_state
from skilgen.core.context import build_codebase_context
from skilgen.core.requirements import load_project_context
from skilgen.core.run_memory import load_current_run_memory
from skilgen.external_skills import (
    activate_external_skill,
    active_external_skills,
    deactivate_external_skill,
    detect_external_skill_sources,
    external_skill_lock,
    external_skill_policy,
    export_external_skill_lock,
    get_external_skill,
    import_external_skill_candidates,
    import_external_skill_lock,
    install_external_skill,
    installed_external_skills,
    list_external_skills,
    ranked_external_skills,
    remove_external_skill,
    sync_all_external_skills,
    sync_external_skill,
)


API_VERSION = "1.0"


def _with_api_meta(payload: dict[str, object]) -> dict[str, object]:
    return {"api_version": API_VERSION, **payload}


def health_payload() -> dict[str, object]:
    return _with_api_meta({"status": "ok", "service": "skilgen", "runtime": current_runtime_mode()})


def doctor_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(runtime_diagnostics(Path(project_root).resolve()))


def decision_payload(project_root: str | Path, requirements: str | Path | None = None) -> dict[str, object]:
    root = Path(project_root).resolve()
    req = Path(requirements).resolve() if requirements is not None else None
    requirements_context = load_project_context(root, req)
    codebase_context = build_codebase_context(root, requirements_context)
    decision = build_agent_decision(root, requirements_context, codebase_context.domain_graph, codebase_context.skill_tree)
    return _with_api_meta(decision.__dict__)


def fingerprint_payload(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "fingerprint",
            f"Fingerprint the project at {root} and return framework_fingerprint JSON.",
            lambda: native_fingerprint_payload(root),
        )
    )


def intent_payload(requirements: str | Path) -> dict[str, object]:
    path = Path(requirements).resolve()
    runtime = DeepAgentsRuntime(path.parent)
    return _with_api_meta(
        runtime.run(
            "intent",
            f"Parse the requirements file at {path} and return structured project intent JSON.",
            lambda: native_intent_payload(path),
        )
    )


def plan_payload(requirements: str | Path | None, project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    req = Path(requirements).resolve() if requirements is not None else None
    runtime = DeepAgentsRuntime(root)
    events = [
        {"message": "Reading project scope and available inputs for roadmap planning."},
        {"message": "Synthesizing implementation phases and sequencing the next delivery steps."},
    ]
    result = runtime.run(
            "plan",
            f"Build a roadmap plan for project_root={root} using requirements={req}. Return JSON with model and steps.",
            lambda: native_plan_payload(req, root),
        )
    return _with_api_meta(
        {
            "runtime": current_runtime_mode(root),
            "runtime_diagnostics": runtime_diagnostics(root),
            "events": events,
            **result,
        }
    )


def analyze_payload(project_root: str | Path, requirements: str | Path | None = None) -> dict[str, object]:
    root = Path(project_root).resolve()
    req = Path(requirements).resolve() if requirements is not None else None
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "analyze",
            f"Analyze project_root={root} requirements={req} and return JSON with project_root, framework_fingerprint, signals, import_graph, and optional detected_domains/skill_tree.",
            lambda: native_analyze_payload(root, req),
        )
    )


def deliver_payload(requirements: str | Path | None, project_root: str | Path) -> dict[str, object]:
    events: list[dict[str, object]] = []

    def report(message: str) -> None:
        events.append({"message": message})

    generated = run_delivery(
        Path(requirements).resolve() if requirements is not None else None,
        Path(project_root).resolve(),
        progress_callback=report,
    )
    return _with_api_meta(
        {
            "runtime": current_runtime_mode(Path(project_root).resolve()),
            "runtime_diagnostics": runtime_diagnostics(Path(project_root).resolve()),
            "events": events,
            "generated_files": [str(path) for path in generated],
        }
    )


def preview_payload(
    requirements: str | Path | None,
    project_root: str | Path,
    *,
    targets: tuple[str, ...] = ("docs", "skills"),
    domains: tuple[str, ...] = (),
) -> dict[str, object]:
    root = Path(project_root).resolve()
    req = Path(requirements).resolve() if requirements is not None else None
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "preview",
            f"Preview delivery for project_root={root} requirements={req} targets={list(targets)} domains={list(domains)} without writing files.",
            lambda: native_preview_payload(req, root, targets=targets, domains=domains),
        )
    )


def create_deliver_job(requirements: str | Path | None, project_root: str | Path) -> dict[str, object]:
    resolved_requirements = str(Path(requirements).resolve()) if requirements is not None else None
    resolved_root = str(Path(project_root).resolve())

    def run_with_progress(report: Callable[[int, str], None]) -> dict[str, object]:
        report(10, "Starting delivery and loading project inputs.")
        report(25, "Reading the codebase and optional requirements.")
        report(45, "Building project context and identifying implementation patterns.")
        report(70, "Generating docs and skills for coding agents.")
        result = deliver_payload(resolved_requirements, resolved_root)
        report(90, "Finalizing generated outputs.")
        return result

    job = submit_job(
        "deliver",
        {"requirements": resolved_requirements, "project_root": resolved_root},
        run_with_progress,
    )
    return _with_api_meta(job_payload(job))


def cancel_job_payload(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    job = request_cancel(job_id, project_root)
    if job is None:
        return _with_api_meta({"error": "not_found", "job_id": job_id})
    return _with_api_meta(job_payload(job))


def resume_job_payload(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    job = get_job(job_id, project_root)
    if job is None:
        return _with_api_meta({"error": "not_found", "job_id": job_id})
    requirements = job.payload.get("requirements")
    resolved_root = job.payload.get("project_root")
    if job.job_type != "deliver" or not isinstance(requirements, str) or not isinstance(resolved_root, str):
        return _with_api_meta({"error": "unsupported_resume", "job_id": job_id})
    if job.status not in {"failed", "cancelled"}:
        return _with_api_meta({"error": "resume_not_allowed", "job_id": job_id, "status": job.status})
    return create_deliver_job(requirements, resolved_root)


def job_status_payload(job_id: str, project_root: str | Path | None = None) -> dict[str, object]:
    job = get_job(job_id, project_root)
    if job is None:
        return _with_api_meta({"error": "not_found", "job_id": job_id})
    return _with_api_meta(job_payload(job))


def jobs_payload(project_root: str | Path | None = None) -> dict[str, object]:
    return _with_api_meta({"jobs": [job_payload(job) for job in list_jobs(project_root)]})


def features_payload(requirements: str | Path | None, project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    req = Path(requirements).resolve() if requirements is not None else None
    runtime = DeepAgentsRuntime(root)
    events = [
        {"message": "Reading the codebase and optional requirements to identify product capabilities."},
        {"message": "Grouping detected backend, frontend, and planning signals into a feature inventory."},
    ]
    result = runtime.run(
            "features",
            f"Build the feature inventory for project_root={root} requirements={req}. Return JSON with features.",
            lambda: native_features_payload(req, root),
        )
    return _with_api_meta(
        {
            "runtime": current_runtime_mode(root),
            "runtime_diagnostics": runtime_diagnostics(root),
            "events": events,
            **result,
        }
    )


def map_payload(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "map",
            f"Build the import and relationship map for project_root={root}. Return JSON with import_graph.",
            lambda: native_map_payload(root),
        )
    )


def status_payload(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    runtime = DeepAgentsRuntime(root)
    requirements_context = load_project_context(root, None)
    codebase_context = build_codebase_context(root, requirements_context)
    freshness = compute_freshness_report(root, requirements_context, codebase_context.domain_graph, load_freshness_state(root))
    current_run = load_current_run_memory(root)
    external_skill_detection = detect_external_skill_sources(root)
    return _with_api_meta(
        {
            **runtime.run(
                "status",
                f"Summarize generated artifact status for project_root={root}.",
                lambda: native_status_payload(root),
            ),
            "freshness": freshness.__dict__,
            "current_run_memory": current_run.__dict__ if current_run is not None else None,
            "agent_decision": build_agent_decision(root, requirements_context, codebase_context.domain_graph, codebase_context.skill_tree).__dict__,
            "installed_external_skills": installed_external_skills(root),
            "active_external_skills": active_external_skills(root),
            "ranked_external_skills": ranked_external_skills(root),
            "external_skill_lock": external_skill_lock(root),
            "external_skill_policy": external_skill_policy(root),
            "external_skill_recommendations": external_skill_detection["manual_recommendations"],
            "enterprise_skills": list_enterprise_skills(root),
            "active_enterprise_skills": active_enterprise_skills(root),
            "recommended_mcp_connectors": recommend_mcp_connectors(root),
            "active_mcp_connectors": active_mcp_connectors(root),
            "auto_update": auto_update_status(root),
        }
    )


def skills_list_payload(project_root: str | Path, ecosystem: str | None = None, search: str | None = None) -> dict[str, object]:
    return _with_api_meta(list_external_skills(Path(project_root).resolve(), ecosystem=ecosystem, search=search))


def skills_detect_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(detect_external_skill_sources(Path(project_root).resolve()))


def skills_active_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta({"skills": active_external_skills(Path(project_root).resolve())})


def skills_lock_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(external_skill_lock(Path(project_root).resolve()))


def skills_lock_export_payload(project_root: str | Path, output_path: str | Path | None = None) -> dict[str, object]:
    return _with_api_meta(export_external_skill_lock(project_root=Path(project_root).resolve(), output_path=output_path))


def skills_lock_import_payload(project_root: str | Path, input_path: str | Path, *, sync_existing: bool = False) -> dict[str, object]:
    return _with_api_meta(import_external_skill_lock(project_root=Path(project_root).resolve(), input_path=input_path, sync_existing=sync_existing))


def skills_rank_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(ranked_external_skills(Path(project_root).resolve()))


def skills_policy_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(external_skill_policy(Path(project_root).resolve()))


def skills_show_payload(slug: str, project_root: str | Path) -> dict[str, object]:
    return _with_api_meta({"skill": get_external_skill(slug, Path(project_root).resolve())})


def skills_install_payload(
    project_root: str | Path,
    *,
    slug: str | None = None,
    git_url: str | None = None,
    name: str | None = None,
    force: bool = False,
    ref: str | None = None,
    active: bool | None = None,
) -> dict[str, object]:
    return _with_api_meta(
        {
            "installed_skill": install_external_skill(
                project_root=Path(project_root).resolve(),
                slug=slug,
                git_url=git_url,
                name=name,
                force=force,
                ref=ref,
                active=active,
            )
        }
    )


def skills_import_payload(
    project_root: str | Path,
    slug: str,
    *,
    limit: int = 5,
    active: bool | None = None,
) -> dict[str, object]:
    return _with_api_meta(import_external_skill_candidates(project_root=Path(project_root).resolve(), slug=slug, limit=limit, active=active))


def skills_sync_payload(project_root: str | Path, slug: str | None = None, *, all_sources: bool = False) -> dict[str, object]:
    root = Path(project_root).resolve()
    if all_sources:
        return _with_api_meta(sync_all_external_skills(project_root=root))
    if slug is None:
        return _with_api_meta({"error": "missing_slug"})
    return _with_api_meta({"synced_skill": sync_external_skill(project_root=root, slug=slug)})


def skills_remove_payload(project_root: str | Path, slug: str) -> dict[str, object]:
    return _with_api_meta({"removed_skill": remove_external_skill(project_root=Path(project_root).resolve(), slug=slug)})


def skills_activate_payload(project_root: str | Path, slug: str) -> dict[str, object]:
    return _with_api_meta({"activated_skill": activate_external_skill(project_root=Path(project_root).resolve(), slug=slug)})


def skills_deactivate_payload(project_root: str | Path, slug: str) -> dict[str, object]:
    return _with_api_meta({"deactivated_skill": deactivate_external_skill(project_root=Path(project_root).resolve(), slug=slug)})


def enterprise_list_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta({"skills": list_enterprise_skills(Path(project_root).resolve())})


def enterprise_ingest_payload(
    project_root: str | Path,
    *,
    name: str,
    path: str | Path | None = None,
    git_url: str | None = None,
    ref: str | None = None,
    activate: bool | None = None,
    kind: str = "enterprise",
) -> dict[str, object]:
    return _with_api_meta(
        {
            "enterprise_skill": ingest_enterprise_skill(
                Path(project_root).resolve(),
                name=name,
                path=path,
                git_url=git_url,
                ref=ref,
                activate=activate,
                kind=kind,
            )
        }
    )


def enterprise_generate_payload(
    project_root: str | Path,
    *,
    name: str,
    source_paths: list[str | Path],
    kind: str = "domain",
    activate: bool = True,
) -> dict[str, object]:
    return _with_api_meta(
        {
            "enterprise_skill": generate_enterprise_skill(
                Path(project_root).resolve(),
                name=name,
                source_paths=source_paths,
                kind=kind,
                activate=activate,
            )
        }
    )


def connectors_list_payload(system: str | None = None, search: str | None = None) -> dict[str, object]:
    return _with_api_meta(connector_catalog(system=system, search=search))


def connectors_recommend_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta(recommend_mcp_connectors(Path(project_root).resolve()))


def connectors_active_payload(project_root: str | Path) -> dict[str, object]:
    return _with_api_meta({"connectors": active_mcp_connectors(Path(project_root).resolve())})


def connectors_activate_payload(project_root: str | Path, slug: str) -> dict[str, object]:
    return _with_api_meta({"connector": activate_mcp_connector(Path(project_root).resolve(), slug)})


def connectors_deactivate_payload(project_root: str | Path, slug: str) -> dict[str, object]:
    return _with_api_meta({"connector": deactivate_mcp_connector(Path(project_root).resolve(), slug)})


def report_payload(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "report",
            f"Build a project report for project_root={root}. Return JSON with status, domains, signal_counts, and summary.",
            lambda: native_report_payload(root),
        )
    )


def validate_payload(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    runtime = DeepAgentsRuntime(root)
    return _with_api_meta(
        runtime.run(
            "validate",
            f"Validate project_root={root}. Return JSON with valid, errors, warnings, coverage, and completeness_score.",
            lambda: native_validate_payload(root),
        )
    )



import json
from pathlib import Path
import subprocess
import sys
from tempfile import TemporaryDirectory
import unittest


class PlanCliTests(unittest.TestCase):
    def test_plan_command_outputs_steps(self) -> None:
        with TemporaryDirectory() as tmp:
            root = Path(tmp)
            requirements = root / "requirements.md"
            requirements.write_text(
                "# Product Requirements\n\n- Add a backend scan endpoint\n- Add a dashboard UI flow\n",
                encoding="utf-8",
            )
            result = subprocess.run(
                [
                    sys.executable,
                    "-m",
                    "skilgen.cli.main",
                    "plan",
                    "--requirements",
                    str(requirements),
                    "--project-root",
                    str(root),
                ],
                text=True,
                capture_output=True,
                check=True,
            )
            payload = json.loads(result.stdout)
            self.assertIn("model", payload)
            self.assertTrue(payload["steps"])


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from pathlib import Path

from skilgen.core.models import FrameworkFingerprint, FrameworkMatch


def _gather_files(project_root: Path) -> set[str]:
    names: set[str] = set()
    for path in project_root.rglob("*"):
        if not path.is_file():
            continue
        if ".skilgen" in path.relative_to(project_root).parts:
            continue
        names.add(path.name)
        names.add(path.as_posix().replace(f"{project_root.as_posix()}/", ""))
    return names


def _match(files: set[str], candidates: list[tuple[str, list[str]]]) -> FrameworkMatch | None:
    best: FrameworkMatch | None = None
    for name, evidence_patterns in candidates:
        matched = [pattern for pattern in evidence_patterns if any(pattern in file for file in files)]
        if not matched:
            continue
        confidence = min(0.99, 0.45 + 0.2 * len(matched))
        candidate = FrameworkMatch(name=name, confidence=confidence, evidence=matched)
        if best is None or candidate.confidence > best.confidence:
            best = candidate
    return best


def fingerprint_project(project_root: Path) -> FrameworkFingerprint:
    files = _gather_files(project_root)
    frontend = _match(
        files,
        [
            ("nextjs", ["next.config", "app/", "pages/"]),
            ("react", ["package.json", "src/", ".tsx", ".jsx"]),
            ("vue", ["vite.config", ".vue"]),
            ("svelte", ["svelte.config", ".svelte"]),
        ],
    )
    backend = _match(
        files,
        [
            ("fastapi", ["fastapi", "main.py", "app/api", "pyproject.toml"]),
            ("django", ["manage.py", "settings.py"]),
            ("flask", ["flask", "wsgi.py"]),
            ("express", ["package.json", "express"]),
        ],
    )
    test_framework = _match(
        files,
        [
            ("unittest", ["test_", "unittest"]),
            ("pytest", ["pytest", "conftest.py"]),
            ("jest", ["jest.config", ".test.ts", ".spec.ts"]),
        ],
    )
    build_tool = _match(
        files,
        [
            ("setuptools", ["pyproject.toml"]),
            ("vite", ["vite.config"]),
            ("webpack", ["webpack.config"]),
        ],
    )
    return FrameworkFingerprint(
        frontend=frontend,
        backend=backend,
        test_framework=test_framework,
        build_tool=build_tool,
    )



from __future__ import annotations

from dataclasses import asdict, dataclass
from datetime import UTC, datetime
import json
import re
import shutil
import subprocess
from pathlib import Path
from skilgen.core.config import load_config


TRUST_SCORES = {
    "official": 5,
    "spec": 4,
    "curated": 3,
    "community": 2,
    "directory": 1,
    "custom": 1,
}


@dataclass(frozen=True)
class ExternalSkillSource:
    slug: str
    name: str
    ecosystem: str
    publisher: str
    description: str
    repository_url: str
    source_path: str | None
    docs_url: str
    category: str = "official"
    trust_level: str = "official"
    supported_agents: tuple[str, ...] = ()
    install_strategy: str = "git_clone"
    tags: tuple[str, ...] = ()


CATALOG: tuple[ExternalSkillSource, ...] = (
    ExternalSkillSource(
        slug="anthropic-skills",
        name="Anthropic Skills",
        ecosystem="anthropic",
        publisher="Anthropic",
        description="Official Anthropic skills collection with reference skills and SKILL template patterns.",
        repository_url="https://github.com/anthropics/skills.git",
        source_path="skills",
        docs_url="https://github.com/anthropics/skills/tree/main/skills",
        supported_agents=("Claude Code",),
        tags=("official", "claude", "skills"),
    ),
    ExternalSkillSource(
        slug="langchain-skills",
        name="LangChain Skills",
        ecosystem="langchain",
        publisher="LangChain AI",
        description="Official LangChain skills for LangChain, LangGraph, Deep Agents, RAG, and orchestration flows.",
        repository_url="https://github.com/langchain-ai/langchain-skills.git",
        source_path=None,
        docs_url="https://github.com/langchain-ai/langchain-skills",
        supported_agents=("Claude Code", "Cursor", "Windsurf", "Codex"),
        tags=("official", "deep-agents", "langchain"),
    ),
    ExternalSkillSource(
        slug="langsmith-skills",
        name="LangSmith Skills",
        ecosystem="langchain",
        publisher="LangChain AI",
        description="Official LangSmith skills for observability, evaluation, prompt engineering, and tracing workflows.",
        repository_url="https://github.com/langchain-ai/langsmith-skills.git",
        source_path=None,
        docs_url="https://github.com/langchain-ai/langsmith-skills",
        supported_agents=("Claude Code", "Cursor", "Codex"),
        tags=("official", "langsmith", "evaluation", "observability"),
    ),
    ExternalSkillSource(
        slug="huggingface-skills",
        name="Hugging Face Skills",
        ecosystem="huggingface",
        publisher="Hugging Face",
        description="Official Hugging Face skills collection for hub, datasets, jobs, trainers, and evaluation workflows.",
        repository_url="https://github.com/huggingface/skills.git",
        source_path=None,
        docs_url="https://github.com/huggingface/skills",
        supported_agents=("Claude Code", "Codex", "Gemini CLI", "Cursor"),
        tags=("official", "huggingface", "skills"),
    ),
    ExternalSkillSource(
        slug="huggingface-upskill",
        name="Hugging Face Upskill",
        ecosystem="huggingface",
        publisher="Hugging Face",
        description="Official Hugging Face tool for auto-generating and benchmarking skills with teacher-student approaches.",
        repository_url="https://github.com/huggingface/upskill.git",
        source_path=None,
        docs_url="https://github.com/huggingface/upskill",
        supported_agents=("Claude Code", "Codex", "Gemini CLI"),
        tags=("official", "huggingface", "generation", "benchmarking"),
    ),
    ExternalSkillSource(
        slug="awesome-copilot",
        name="Awesome Copilot",
        ecosystem="github",
        publisher="GitHub / Microsoft",
        description="Official GitHub Copilot skill and workflow examples for Azure, AWS, BigQuery, IAM, and VS Code.",
        repository_url="https://github.com/github/awesome-copilot.git",
        source_path=None,
        docs_url="https://github.com/github/awesome-copilot",
        supported_agents=("GitHub Copilot", "VS Code"),
        tags=("official", "copilot", "workflows"),
    ),
    ExternalSkillSource(
        slug="agentskills-spec",
        name="AgentSkills Spec",
        ecosystem="spec",
        publisher="agentskills.io",
        description="Open standard specification and documentation for the SKILL.md format.",
        repository_url="https://github.com/agentskills/agentskills.git",
        source_path=None,
        docs_url="https://github.com/agentskills/agentskills",
        category="spec",
        trust_level="spec",
        supported_agents=("All agents",),
        tags=("spec", "skill-md", "standard"),
    ),
    ExternalSkillSource(
        slug="n8n-mcp-patterns",
        name="n8n MCP Patterns",
        ecosystem="n8n",
        publisher="czlonkowski",
        description="n8n skills for code nodes, expressions, MCP tooling, workflow patterns, and validation.",
        repository_url="https://github.com/czlonkowski/n8n-mcp.git",
        source_path=None,
        docs_url="https://github.com/czlonkowski/n8n-mcp",
        category="framework",
        trust_level="community",
        supported_agents=("Claude Code", "Cursor", "Windsurf"),
        tags=("n8n", "mcp", "workflow"),
    ),
    ExternalSkillSource(
        slug="ai-research-skills",
        name="AI Research Skills",
        ecosystem="multi-framework",
        publisher="Orchestra Research",
        description="83+ research and coding skills for LangChain, LlamaIndex, CrewAI, fine-tuning, RLHF, and RAG.",
        repository_url="https://github.com/Orchestra-Research/AI-Research-SKILLS.git",
        source_path=None,
        docs_url="https://github.com/Orchestra-Research/AI-Research-SKILLS",
        category="framework",
        trust_level="community",
        supported_agents=("Claude Code", "Codex", "Cursor", "Gemini CLI"),
        tags=("research", "multi-framework", "rag"),
    ),
    ExternalSkillSource(
        slug="context-engineering-skills",
        name="Context Engineering Skills",
        ecosystem="framework",
        publisher="muratcankoylan",
        description="Context engineering, multi-agent architecture, memory patterns, and LLM-as-judge skills.",
        repository_url="https://github.com/muratcankoylan/Agent-Skills-for-Context-Engineering.git",
        source_path=None,
        docs_url="https://github.com/muratcankoylan/Agent-Skills-for-Context-Engineering",
        category="framework",
        trust_level="community",
        supported_agents=("Claude Code", "Codex", "Gemini CLI"),
        tags=("context-engineering", "memory", "multi-agent"),
    ),
    ExternalSkillSource(
        slug="skill-seekers",
        name="Skill Seekers",
        ecosystem="tooling",
        publisher="yusufkaraaslan",
        description="Tooling to convert docs, sites, repos, and PDFs into SKILL.md with RAG assets.",
        repository_url="https://github.com/yusufkaraaslan/Skill_Seekers.git",
        source_path=None,
        docs_url="https://github.com/yusufkaraaslan/Skill_Seekers",
        category="tooling",
        trust_level="community",
        supported_agents=("All agents",),
        tags=("tooling", "conversion", "rag"),
    ),
    ExternalSkillSource(
        slug="awesome-agent-skills-voltagent",
        name="Awesome Agent Skills (VoltAgent)",
        ecosystem="directory",
        publisher="VoltAgent",
        description="Large directory of official and community agent skills across Anthropic, Codex, LangChain, and n8n.",
        repository_url="https://github.com/VoltAgent/awesome-agent-skills.git",
        source_path=None,
        docs_url="https://github.com/VoltAgent/awesome-agent-skills",
        category="directory",
        trust_level="directory",
        supported_agents=("Claude Code", "Codex", "Gemini CLI", "Cursor"),
        tags=("directory", "aggregator", "awesome-list"),
    ),
    ExternalSkillSource(
        slug="awesome-agent-skills-skillmatic",
        name="Awesome Agent Skills (skillmatic-ai)",
        ecosystem="directory",
        publisher="skillmatic-ai",
        description="Directory of agent skills, authoring guides, scanners, and marketplace-style discovery links.",
        repository_url="https://github.com/skillmatic-ai/awesome-agent-skills.git",
        source_path=None,
        docs_url="https://github.com/skillmatic-ai/awesome-agent-skills",
        category="directory",
        trust_level="directory",
        supported_agents=("All agents",),
        tags=("directory", "aggregator", "marketplace"),
    ),
    ExternalSkillSource(
        slug="awesome-agent-skills-heilcheng",
        name="Awesome Agent Skills (heilcheng)",
        ecosystem="directory",
        publisher="heilcheng",
        description="Directory of skills for Claude, Codex, GitHub Copilot, VS Code, and Google Workspace agent workflows.",
        repository_url="https://github.com/heilcheng/awesome-agent-skills.git",
        source_path=None,
        docs_url="https://github.com/heilcheng/awesome-agent-skills",
        category="directory",
        trust_level="directory",
        supported_agents=("Claude Code", "Codex", "Copilot"),
        tags=("directory", "workspace", "copilot"),
    ),
    ExternalSkillSource(
        slug="awesome-llm-skills",
        name="Awesome LLM Skills",
        ecosystem="directory",
        publisher="Prat011",
        description="Directory covering Notion, Google Workspace, multi-agent, Gemini CLI, OpenCode, and Qwen-ready skills.",
        repository_url="https://github.com/Prat011/awesome-llm-skills.git",
        source_path=None,
        docs_url="https://github.com/Prat011/awesome-llm-skills",
        category="directory",
        trust_level="directory",
        supported_agents=("All agents",),
        tags=("directory", "multi-agent", "workspace"),
    ),
    ExternalSkillSource(
        slug="curated-ai-agent-skills",
        name="Curated AI Agent Skills",
        ecosystem="curated",
        publisher="MoizIbnYousaf",
        description="Curated personal library with trust metadata, provenance, and universal installation ideas.",
        repository_url="https://github.com/MoizIbnYousaf/AI-Agent-Skills.git",
        source_path=None,
        docs_url="https://github.com/MoizIbnYousaf/AI-Agent-Skills",
        category="curated",
        trust_level="curated",
        supported_agents=("Claude Code", "Codex", "cross-agent"),
        tags=("curated", "provenance", "trust"),
    ),
    ExternalSkillSource(
        slug="skills-benchmarks",
        name="Skills Benchmarks",
        ecosystem="benchmarks",
        publisher="LangChain AI",
        description="Benchmark suite for measuring skill quality and performance across LangChain and LangSmith tasks.",
        repository_url="https://github.com/langchain-ai/skills-benchmarks.git",
        source_path=None,
        docs_url="https://github.com/langchain-ai/skills-benchmarks",
        category="benchmarks",
        trust_level="official",
        supported_agents=("Claude Code", "Codex"),
        tags=("benchmarks", "langchain", "evaluation"),
    ),
)


def _external_skills_root(project_root: str | Path) -> Path:
    return Path(project_root).resolve() / ".skilgen" / "external-skills"


def _manifest_path(project_root: str | Path) -> Path:
    return _external_skills_root(project_root) / "manifest.json"


def _lock_path(project_root: str | Path) -> Path:
    return _external_skills_root(project_root) / "lock.json"


def _sources_dir(project_root: str | Path) -> Path:
    return _external_skills_root(project_root) / "sources"


def _normalized_dir(project_root: str | Path) -> Path:
    return _external_skills_root(project_root) / "normalized"


def _normalize_slug(name: str) -> str:
    return "-".join(part for part in "".join(ch.lower() if ch.isalnum() else "-" for ch in name).split("-") if part)


def _load_manifest(project_root: str | Path) -> dict[str, object]:
    path = _manifest_path(project_root)
    if not path.exists():
        return {"skills": []}
    return json.loads(path.read_text(encoding="utf-8"))


def _write_manifest(project_root: str | Path, payload: dict[str, object]) -> None:
    root = _external_skills_root(project_root)
    root.mkdir(parents=True, exist_ok=True)
    _manifest_path(project_root).write_text(json.dumps(payload, indent=2), encoding="utf-8")


def _load_lock(project_root: str | Path) -> dict[str, object]:
    path = _lock_path(project_root)
    if not path.exists():
        return {"skills": []}
    return json.loads(path.read_text(encoding="utf-8"))


def _write_lock(project_root: str | Path, payload: dict[str, object]) -> None:
    root = _external_skills_root(project_root)
    root.mkdir(parents=True, exist_ok=True)
    _lock_path(project_root).write_text(json.dumps(payload, indent=2), encoding="utf-8")


def _export_lock_path(project_root: str | Path) -> Path:
    return _external_skills_root(project_root) / "export-lock.json"


def _installed_by_slug(project_root: str | Path) -> dict[str, dict[str, object]]:
    manifest = _load_manifest(project_root)
    entries = manifest.get("skills", [])
    if not isinstance(entries, list):
        return {}
    return {
        str(entry.get("slug")): entry
        for entry in entries
        if isinstance(entry, dict) and entry.get("slug")
    }


def _lock_by_slug(project_root: str | Path) -> dict[str, dict[str, object]]:
    lock = _load_lock(project_root)
    entries = lock.get("skills", [])
    if not isinstance(entries, list):
        return {}
    return {
        str(entry.get("slug")): entry
        for entry in entries
        if isinstance(entry, dict) and entry.get("slug")
    }


def _catalog_entry(slug: str) -> ExternalSkillSource | None:
    for entry in CATALOG:
        if entry.slug == slug:
            return entry
    return None


def _serialize_entry(entry: ExternalSkillSource, installed: dict[str, object] | None) -> dict[str, object]:
    payload = asdict(entry)
    payload["installed"] = installed is not None
    payload["install_path"] = installed.get("install_path") if installed else None
    payload["installed_metadata"] = installed
    payload["trust_score"] = TRUST_SCORES.get(entry.trust_level, 0)
    return payload


def installed_external_skills(project_root: str | Path = ".") -> list[dict[str, object]]:
    return sorted(_installed_by_slug(project_root).values(), key=lambda entry: str(entry.get("slug", "")))


def external_skill_lock(project_root: str | Path = ".") -> dict[str, object]:
    return _load_lock(project_root)


def active_external_skills(project_root: str | Path = ".") -> list[dict[str, object]]:
    installed = _installed_by_slug(project_root)
    locked = _lock_by_slug(project_root)
    active: list[dict[str, object]] = []
    for slug, entry in installed.items():
        lock_entry = locked.get(slug, {})
        if lock_entry.get("active"):
            active.append({**entry, "lock_metadata": lock_entry})
    return sorted(active, key=lambda entry: str(entry.get("slug", "")))


def _repo_keyword_profile(project_root: str | Path) -> set[str]:
    _, text = _repo_text_snapshot(Path(project_root).resolve())
    keywords: set[str] = set()
    keyword_map = {
        "claude": ("claude", "anthropic"),
        "langchain": ("langchain", "langgraph", "deep agents", "deepagents"),
        "langsmith": ("langsmith", "evaluation", "observability", "tracing"),
        "huggingface": ("huggingface", "hugging face", "transformers", "datasets", "diffusers"),
        "copilot": ("copilot", "github copilot", "copilot-instructions"),
        "n8n": ("n8n", "workflow"),
        "research": ("rag", "crewai", "llamaindex", "rlhf", "fine-tuning", "finetuning"),
        "benchmark": ("benchmark", "benchmarks"),
        "context": ("memory", "multi-agent", "context engineering"),
    }
    for label, needles in keyword_map.items():
        if any(needle in text for needle in needles):
            keywords.add(label)
    return keywords


def _catalog_tags(entry: ExternalSkillSource) -> set[str]:
    values = {
        entry.ecosystem.lower(),
        entry.publisher.lower(),
        entry.category.lower(),
        entry.trust_level.lower(),
        *(tag.lower() for tag in entry.tags),
        *(agent.lower() for agent in entry.supported_agents),
    }
    return {value for value in values if value}


def prioritized_active_external_skills(project_root: str | Path = ".") -> list[dict[str, object]]:
    root = Path(project_root).resolve()
    active = active_external_skills(root)
    detection_map = {
        str(entry["slug"]): entry for entry in detect_external_skill_sources(root).get("detected_skills", [])
    }
    keywords = _repo_keyword_profile(root)
    ranked: list[dict[str, object]] = []
    for entry in active:
        slug = str(entry["slug"])
        lock_metadata = entry.get("lock_metadata", {}) if isinstance(entry.get("lock_metadata"), dict) else {}
        source = _catalog_entry(slug)
        tags = _catalog_tags(source) if source is not None else set()
        matched_keywords = sorted(keyword for keyword in keywords if keyword in tags)
        detection = detection_map.get(slug)
        trust_level = str(entry.get("trust_level", source.trust_level if source else "custom"))
        trust_score = TRUST_SCORES.get(trust_level, 0)
        score = trust_score * 10
        score += len(matched_keywords) * 5
        if detection is not None:
            score += 12
        normalized = lock_metadata.get("normalized", {}) if isinstance(lock_metadata, dict) else {}
        score += min(int(normalized.get("entry_count", 0) or 0), 20)
        ranked.append(
            {
                **entry,
                "priority_score": score,
                "priority_reason": (
                    detection["reasons"][0]
                    if isinstance(detection, dict) and detection.get("reasons")
                    else f"Trust level `{trust_level}` and ecosystem fit for this repo."
                ),
                "matched_keywords": matched_keywords,
                "trust_score": trust_score,
            }
        )
    return sorted(ranked, key=lambda item: (-int(item["priority_score"]), str(item["slug"])))


def _repo_text_snapshot(project_root: Path) -> tuple[list[str], str]:
    ignored_parts = {".git", ".venv", "venv", "node_modules", "__pycache__", "dist", "build", ".next", ".skilgen", "skills"}
    ignored_files = {"AGENTS.md", "ANALYSIS.md", "FEATURES.md", "REPORT.md", "TRACEABILITY.md"}
    text_extensions = {".py", ".ts", ".tsx", ".js", ".jsx", ".json", ".toml", ".yml", ".yaml", ".md", ".txt"}
    paths: list[str] = []
    snippets: list[str] = []
    for path in sorted(project_root.rglob("*")):
        if not path.is_file():
            continue
        if path.name in ignored_files:
            continue
        relative = path.relative_to(project_root)
        if set(relative.parts) & ignored_parts:
            continue
        rel = relative.as_posix()
        paths.append(rel.lower())
        if path.suffix.lower() in text_extensions and len(snippets) < 80:
            try:
                snippets.append(path.read_text(encoding="utf-8", errors="ignore")[:12000].lower())
            except OSError:
                continue
    return paths, "\n".join(snippets)


def detect_external_skill_sources(project_root: str | Path = ".") -> dict[str, object]:
    root = Path(project_root).resolve()
    lower_paths, text = _repo_text_snapshot(root)
    installed = _installed_by_slug(root)
    detected: list[dict[str, object]] = []
    manual_recommendations: list[dict[str, object]] = []

    def has_path(fragment: str) -> bool:
        needle = fragment.lower()
        return any(needle in path for path in lower_paths)

    def has_text(*needles: str) -> bool:
        return any(needle.lower() in text for needle in needles)

    def has_pattern(pattern: str) -> bool:
        return re.search(pattern, text, re.IGNORECASE) is not None

    def add_detected(slug: str, reason: str) -> None:
        entry = _catalog_entry(slug)
        if entry is None:
            return
        for existing in detected:
            if existing["slug"] == slug:
                existing["reasons"].append(reason)
                return
        detected.append(
            {
                "slug": slug,
                "name": entry.name,
                "ecosystem": entry.ecosystem,
                "installed": slug in installed,
                "reasons": [reason],
            }
        )

    def add_manual(slug: str, reason: str) -> None:
        entry = _catalog_entry(slug)
        if entry is None:
            return
        for existing in manual_recommendations:
            if existing["slug"] == slug:
                existing["reasons"].append(reason)
                return
        manual_recommendations.append(
            {
                "slug": slug,
                "name": entry.name,
                "ecosystem": entry.ecosystem,
                "reasons": [reason],
            }
        )

    if has_path("claude.md") or (root / ".claude").exists() or has_text("claude code", "anthropic"):
        add_detected("anthropic-skills", "Detected Claude/Anthropic repo hints.")
    if has_text("langchain", "langgraph", "deepagents", "deep agents", "langchain_openai", "langchain_anthropic"):
        add_detected("langchain-skills", "Detected LangChain/LangGraph/Deep Agents dependencies.")
    if has_text("langsmith", "langsmith_project", "langsmith tracing", "langsmith sdk"):
        add_detected("langsmith-skills", "Detected LangSmith observability or tracing usage.")
    if has_text("huggingface_hub", "transformers", "diffusers", "datasets", "trl", "sentence_transformers", "hugging face"):
        add_detected("huggingface-skills", "Detected Hugging Face package usage.")
        if has_text("benchmark", "teacher", "student", "evaluation", "trainer", "upskill"):
            add_detected("huggingface-upskill", "Detected Hugging Face evaluation or teacher/student workflow hints.")
    if has_path(".github/copilot-instructions.md") or (root / ".copilot").exists() or has_text("github copilot", "copilot-instructions"):
        add_detected("awesome-copilot", "Detected GitHub Copilot instructions or workspace setup.")
    if has_text("n8n-nodes", "n8n workflow") or has_path("n8n"):
        add_detected("n8n-mcp-patterns", "Detected n8n workflow or MCP-style repo signals.")
    if has_text("llamaindex", "crewai", "rlhf", "fine-tuning", "finetuning") or (
        has_text("pinecone", "qdrant", "chroma") and has_pattern(r"\brag\b")
    ):
        add_detected("ai-research-skills", "Detected research-stack or retrieval workflow dependencies.")
    if has_text("context engineering", "llm-as-judge", "memory strategy") or (
        has_pattern(r"\bmulti-agent\b") and has_pattern(r"\bmemory\b")
    ):
        add_detected("context-engineering-skills", "Detected context engineering or multi-agent memory patterns.")
    if has_path("skill.md") or has_path("skills/"):
        add_detected("agentskills-spec", "Detected SKILL.md-style files or an existing skills tree.")

    if any(entry["slug"] in {"langchain-skills", "langsmith-skills"} for entry in detected):
        add_manual("skills-benchmarks", "Recommended because LangChain/LangSmith was detected.")
    if detected:
        add_manual("awesome-agent-skills-voltagent", "Recommended directory of adjacent agent skills.")
        add_manual("awesome-agent-skills-skillmatic", "Recommended directory of adjacent agent skills.")
        add_manual("awesome-agent-skills-heilcheng", "Recommended directory of adjacent agent skills.")
        add_manual("awesome-llm-skills", "Recommended directory of adjacent agent skills.")
        add_manual("curated-ai-agent-skills", "Recommended curated cross-agent collection.")
        add_manual("skill-seekers", "Recommended tooling for converting docs and repos into skills.")

    return {
        "detected_skills": sorted(detected, key=lambda entry: entry["slug"]),
        "manual_recommendations": sorted(manual_recommendations, key=lambda entry: entry["slug"]),
        "installed_skills": installed_external_skills(root),
        "auto_install_count": len(detected),
    }


def _git_revision(install_path: Path) -> str | None:
    try:
        result = subprocess.run(
            ["git", "-C", str(install_path), "rev-parse", "HEAD"],
            check=True,
            capture_output=True,
            text=True,
        )
        return result.stdout.strip() or None
    except subprocess.CalledProcessError:
        return None


def _git_remote_url(install_path: Path) -> str | None:
    try:
        result = subprocess.run(
            ["git", "-C", str(install_path), "remote", "get-url", "origin"],
            check=True,
            capture_output=True,
            text=True,
        )
        return result.stdout.strip() or None
    except subprocess.CalledProcessError:
        return None


def _detect_license(install_path: Path) -> dict[str, str] | None:
    for name in ("LICENSE", "LICENSE.md", "LICENSE.txt", "LICENCE", "LICENCE.md", "COPYING"):
        path = install_path / name
        if not path.exists():
            continue
        try:
            first_line = next(
                (line.strip() for line in path.read_text(encoding="utf-8", errors="ignore").splitlines() if line.strip()),
                "",
            )
        except OSError:
            first_line = ""
        return {
            "path": path.relative_to(install_path).as_posix(),
            "summary": first_line or "License file present",
        }
    return None


def _readme_summary(install_path: Path) -> dict[str, str] | None:
    for name in ("README.md", "README.txt", "README"):
        path = install_path / name
        if not path.exists():
            continue
        try:
            lines = [line.strip() for line in path.read_text(encoding="utf-8", errors="ignore").splitlines()]
        except OSError:
            continue
        title = next((line.removeprefix("#").strip() for line in lines if line.startswith("#")), "")
        summary = next(
            (
                line
                for line in lines
                if line
                and not line.startswith("#")
                and not line.startswith("![")
                and not line.startswith("[!")
                and len(line) > 20
            ),
            "",
        )
        if title or summary:
            return {"path": path.relative_to(install_path).as_posix(), "title": title, "summary": summary}
    return None


def _compute_trust_score(
    trust_level: str,
    *,
    license_info: dict[str, str] | None,
    readme_info: dict[str, str] | None,
    entry_count: int,
    repo_candidate_count: int = 0,
) -> int:
    score = TRUST_SCORES.get(trust_level, 0)
    if license_info is not None:
        score += 1
    if readme_info is not None and (readme_info.get("title") or readme_info.get("summary")):
        score += 1
    if entry_count >= 3:
        score += 1
    if repo_candidate_count:
        score += 1
    return min(score, 10)


def _build_provenance_payload(
    *,
    source: ExternalSkillSource,
    install_path: Path,
    requested_ref: str | None,
    resolved_revision: str | None,
    remote_url: str | None,
    license_info: dict[str, str] | None,
    readme_info: dict[str, str] | None,
    install_mode: str,
    imported_from: str | None,
) -> dict[str, object]:
    return {
        "source_slug": source.slug,
        "publisher": source.publisher,
        "repository_url": remote_url or source.repository_url,
        "docs_url": source.docs_url,
        "requested_ref": requested_ref,
        "resolved_revision": resolved_revision,
        "license": license_info,
        "readme": readme_info,
        "install_mode": install_mode,
        "imported_from": imported_from,
        "captured_at": datetime.now(UTC).isoformat(),
        "install_path": str(install_path),
    }


def _adapter_for_source(source: ExternalSkillSource) -> str:
    if source.slug == "anthropic-skills":
        return "anthropic-skills"
    if source.slug in {"langchain-skills", "langsmith-skills"}:
        return "langchain-skills"
    if source.ecosystem == "huggingface":
        return "huggingface-skills"
    if source.category in {"directory", "curated"}:
        return "catalog-directory"
    if source.category == "spec":
        return "skill-spec"
    if source.category == "benchmarks":
        return "benchmarks"
    return "generic-repo"


def _title_from_path(path: Path) -> str:
    return path.stem.replace("-", " ").replace("_", " ")


def _entry_score(adapter: str, relative: str, entry_type: str) -> tuple[int, int]:
    lower = relative.lower()
    entrypoint_score = 0
    keyword_score = 0
    if entry_type == "skill":
        entrypoint_score += 40
    elif entry_type == "readme":
        entrypoint_score += 20
    elif entry_type == "doc":
        entrypoint_score += 5

    if adapter == "anthropic-skills":
        if "/skills/" in f"/{lower}" or lower.startswith("skills/"):
            keyword_score += 30
        if lower.endswith("/skill.md"):
            keyword_score += 20
        if "template" in lower:
            keyword_score += 10
    elif adapter == "langchain-skills":
        if any(token in lower for token in ("langgraph", "deep-agent", "deep_agents", "langsmith", "rag")):
            keyword_score += 25
        if lower.endswith("readme.md"):
            keyword_score += 10
    elif adapter == "huggingface-skills":
        if any(token in lower for token in ("dataset", "trainer", "evaluation", "hub", "benchmark")):
            keyword_score += 25
    elif adapter == "catalog-directory":
        if "awesome" in lower or "index" in lower:
            keyword_score += 20
    elif adapter == "skill-spec":
        if "skill" in lower or "spec" in lower:
            keyword_score += 30
    elif adapter == "benchmarks":
        if "benchmark" in lower or "eval" in lower:
            keyword_score += 30
    elif adapter == "generic-repo":
        if lower.endswith("/skill.md") or lower.endswith("readme.md"):
            keyword_score += 10

    depth_penalty = lower.count("/")
    return entrypoint_score + keyword_score - depth_penalty, depth_penalty


def _collect_normalized_entries(
    install_path: Path,
    source_path: str | None,
    *,
    adapter: str,
) -> list[dict[str, object]]:
    root = install_path / source_path if source_path else install_path
    if not root.exists():
        root = install_path
    entries: list[dict[str, object]] = []
    for path in sorted(root.rglob("*")):
        if not path.is_file():
            continue
        relative = path.relative_to(install_path).as_posix()
        lower = path.name.lower()
        if lower == "skill.md":
            entries.append(
                {
                    "path": relative,
                    "type": "skill",
                    "title": path.parent.name,
                    "entrypoint": True,
                    "adapter_hint": adapter,
                }
            )
        elif lower == "readme.md":
            entries.append(
                {
                    "path": relative,
                    "type": "readme",
                    "title": path.parent.name if path.parent != install_path else path.stem,
                    "entrypoint": len(path.relative_to(root).parts) <= 2,
                    "adapter_hint": adapter,
                }
            )
        elif path.suffix.lower() == ".md" and len(entries) < 50:
            entries.append(
                {
                    "path": relative,
                    "type": "doc",
                    "title": _title_from_path(path),
                    "entrypoint": False,
                    "adapter_hint": adapter,
                }
            )
    decorated = [
        (
            *_entry_score(adapter, str(entry["path"]), str(entry["type"])),
            entry,
        )
        for entry in entries
    ]
    decorated.sort(key=lambda item: (-item[0], item[1], str(item[2]["path"])))
    return [entry for _, _, entry in decorated]


def _extract_github_repo_candidates(install_path: Path) -> list[dict[str, str]]:
    candidates: dict[str, dict[str, str]] = {}
    pattern = re.compile(r"https://github\.com/([A-Za-z0-9_.-]+)/([A-Za-z0-9_.-]+)")
    for readme_name in ("README.md", "README.txt", "README"):
        readme_path = install_path / readme_name
        if not readme_path.exists():
            continue
        try:
            text = readme_path.read_text(encoding="utf-8", errors="ignore")
        except OSError:
            continue
        for owner, repo in pattern.findall(text):
            slug = f"{owner}/{repo}".rstrip(".git")
            url = f"https://github.com/{slug}"
            if url.rstrip("/") == str(readme_path):
                continue
            if slug.lower().startswith(("langchain-ai/awesome", "voltagent/awesome", "skillmatic-ai/awesome", "heilcheng/awesome", "prat011/awesome")):
                continue
            candidates.setdefault(
                slug.lower(),
                {
                    "repo": slug,
                    "url": url,
                },
            )
    return sorted(candidates.values(), key=lambda item: item["repo"])[:50]


def _group_entries_by_adapter(entries: list[dict[str, object]], adapter: str) -> list[dict[str, object]]:
    grouped: dict[str, list[str]] = {}

    def add(group: str, path: str) -> None:
        grouped.setdefault(group, [])
        if path not in grouped[group]:
            grouped[group].append(path)

    for entry in entries:
        path = str(entry["path"])
        lower = path.lower()
        if adapter == "anthropic-skills":
            parts = path.split("/")
            group = parts[1] if len(parts) > 1 and parts[0] == "skills" else parts[0]
            if "template" in lower:
                group = "templates"
            add(group or "root", path)
        elif adapter == "langchain-skills":
            if "langsmith" in lower:
                add("langsmith", path)
            elif "langgraph" in lower:
                add("langgraph", path)
            elif "deep-agent" in lower or "deep_agents" in lower:
                add("deep-agents", path)
            elif "rag" in lower:
                add("rag", path)
            else:
                add("langchain-core", path)
        elif adapter == "huggingface-skills":
            if "dataset" in lower:
                add("datasets", path)
            elif "trainer" in lower or "train" in lower:
                add("training", path)
            elif "eval" in lower or "benchmark" in lower:
                add("evaluation", path)
            elif "hub" in lower:
                add("hub", path)
            else:
                add("general", path)
        elif adapter == "catalog-directory":
            if "awesome" in lower:
                add("directory-index", path)
            elif "skill" in lower:
                add("skill-guides", path)
            else:
                add("references", path)
        elif adapter == "skill-spec":
            if "spec" in lower:
                add("spec", path)
            else:
                add("reference", path)
        elif adapter == "benchmarks":
            if "eval" in lower:
                add("evaluation", path)
            elif "benchmark" in lower:
                add("benchmarks", path)
            else:
                add("supporting-docs", path)
        else:
            add("general", path)

    return [
        {"group": group, "paths": paths[:20], "count": len(paths)}
        for group, paths in sorted(grouped.items())
    ]


def _extract_anthropic_native_view(entries: list[dict[str, object]]) -> dict[str, object]:
    skill_families: dict[str, list[str]] = {}
    templates: list[str] = []
    for entry in entries:
        path = str(entry["path"])
        parts = path.split("/")
        lower = path.lower()
        if "template" in lower:
            templates.append(path)
        if len(parts) >= 3 and parts[0] == "skills" and parts[-1].lower() == "skill.md":
            family = parts[1]
            skill_families.setdefault(family, []).append(path)
    return {
        "skill_families": [
            {"name": name, "paths": sorted(paths)[:12], "count": len(paths)}
            for name, paths in sorted(skill_families.items())
        ],
        "templates": sorted(templates)[:12],
        "template_count": len(templates),
    }


def _extract_langchain_native_view(entries: list[dict[str, object]]) -> dict[str, object]:
    families: dict[str, list[str]] = {
        "langchain-core": [],
        "langgraph": [],
        "deep-agents": [],
        "langsmith": [],
        "rag": [],
    }
    for entry in entries:
        path = str(entry["path"])
        lower = path.lower()
        if "langsmith" in lower:
            families["langsmith"].append(path)
        elif "langgraph" in lower:
            families["langgraph"].append(path)
        elif "deep-agent" in lower or "deep_agents" in lower:
            families["deep-agents"].append(path)
        elif "rag" in lower:
            families["rag"].append(path)
        else:
            families["langchain-core"].append(path)
    return {
        "families": [
            {"name": name, "paths": sorted(paths)[:12], "count": len(paths)}
            for name, paths in families.items()
            if paths
        ],
        "recommended_entry_family": max(
            (
                {"name": name, "count": len(paths)}
                for name, paths in families.items()
                if paths
            ),
            key=lambda item: item["count"],
            default=None,
        ),
    }


def _extract_huggingface_native_view(entries: list[dict[str, object]]) -> dict[str, object]:
    task_families: dict[str, list[str]] = {
        "hub": [],
        "datasets": [],
        "training": [],
        "evaluation": [],
        "general": [],
    }
    for entry in entries:
        path = str(entry["path"])
        lower = path.lower()
        if "dataset" in lower:
            task_families["datasets"].append(path)
        elif "trainer" in lower or "train" in lower:
            task_families["training"].append(path)
        elif "eval" in lower or "benchmark" in lower:
            task_families["evaluation"].append(path)
        elif "hub" in lower:
            task_families["hub"].append(path)
        else:
            task_families["general"].append(path)
    return {
        "task_families": [
            {"name": name, "paths": sorted(paths)[:12], "count": len(paths)}
            for name, paths in task_families.items()
            if paths
        ],
        "recommended_task_family": max(
            (
                {"name": name, "count": len(paths)}
                for name, paths in task_families.items()
                if paths
            ),
            key=lambda item: item["count"],
            default=None,
        ),
    }


def _extract_directory_native_view(entries: list[dict[str, object]], repo_candidates: list[dict[str, str]]) -> dict[str, object]:
    references = [str(entry["path"]) for entry in entries if str(entry["type"]) in {"readme", "doc"}]
    return {
        "reference_docs": references[:20],
        "repo_candidate_count": len(repo_candidates),
        "top_repo_candidates": repo_candidates[:12],
    }


def _extract_adapter_native_view(
    adapter: str,
    entries: list[dict[str, object]],
    repo_candidates: list[dict[str, str]],
) -> dict[str, object]:
    if adapter == "anthropic-skills":
        return _extract_anthropic_native_view(entries)
    if adapter == "langchain-skills":
        return _extract_langchain_native_view(entries)
    if adapter == "huggingface-skills":
        return _extract_huggingface_native_view(entries)
    if adapter == "catalog-directory":
        return _extract_directory_native_view(entries, repo_candidates)
    return {}


def _normalize_external_skill_install(
    project_root: str | Path,
    *,
    source: ExternalSkillSource,
    install_path: Path,
) -> dict[str, object]:
    normalized_root = _normalized_dir(project_root) / source.slug
    normalized_root.mkdir(parents=True, exist_ok=True)
    adapter = _adapter_for_source(source)
    entries = _collect_normalized_entries(install_path, source.source_path, adapter=adapter)
    groups = _group_entries_by_adapter(entries, adapter)
    entry_type_counts: dict[str, int] = {}
    for entry in entries:
        entry_type_counts[str(entry["type"])] = entry_type_counts.get(str(entry["type"]), 0) + 1
    license_info = _detect_license(install_path)
    readme_info = _readme_summary(install_path)
    repo_candidates = _extract_github_repo_candidates(install_path) if adapter == "catalog-directory" else []
    native_view = _extract_adapter_native_view(adapter, entries, repo_candidates)
    trust_score = _compute_trust_score(
        source.trust_level,
        license_info=license_info,
        readme_info=readme_info,
        entry_count=len(entries),
        repo_candidate_count=len(repo_candidates),
    )
    payload = {
        "slug": source.slug,
        "adapter": adapter,
        "ecosystem": source.ecosystem,
        "publisher": source.publisher,
        "trust_level": source.trust_level,
        "trust_score": trust_score,
        "supported_agents": list(source.supported_agents),
        "repository_url": source.repository_url,
        "docs_url": source.docs_url,
        "entry_count": len(entries),
        "entry_type_counts": entry_type_counts,
        "entrypoints": [entry["path"] for entry in entries if entry["entrypoint"]][:12],
        "groups": groups,
        "license": license_info,
        "readme": readme_info,
        "repo_candidates": repo_candidates,
        "native_view": native_view,
        "entries": entries[:100],
    }
    index_path = normalized_root / "index.json"
    index_path.write_text(json.dumps(payload, indent=2), encoding="utf-8")
    summary_path = normalized_root / "SUMMARY.md"
    entrypoint_lines = [f"- `{path}`" for path in payload["entrypoints"]] or [
        "- No obvious entrypoints were indexed."
    ]
    summary_lines = [
        f"# {source.name}",
        "",
        f"- Adapter: `{payload['adapter']}`",
        f"- Ecosystem: `{payload['ecosystem']}`",
        f"- Publisher: `{payload['publisher']}`",
        f"- Trust level: `{payload['trust_level']}` (score: {payload['trust_score']})",
        f"- Repository: {payload['repository_url']}",
        f"- Docs: {payload['docs_url']}",
        f"- License: {(license_info or {}).get('summary', 'unknown')}",
        f"- README summary: {(readme_info or {}).get('summary', 'none')}",
        f"- Entrypoints indexed: {len(payload['entrypoints'])}",
        f"- Entry types: {', '.join(f'{name}={count}' for name, count in sorted(entry_type_counts.items())) or 'none'}",
        f"- Supported agents: {', '.join(payload['supported_agents']) or 'unknown'}",
        "",
        "## Entrypoints",
        *entrypoint_lines,
    ]
    if groups:
        summary_lines.extend(["", "## Groups"])
        summary_lines.extend(
            f"- `{group['group']}` ({group['count']}): {', '.join(f'`{path}`' for path in group['paths'][:4])}"
            for group in groups[:12]
        )
    if repo_candidates:
        summary_lines.extend(["", "## Downstream Repo Candidates"])
        summary_lines.extend(
            f"- `{candidate['repo']}`: {candidate['url']}"
            for candidate in repo_candidates[:12]
        )
    if native_view:
        summary_lines.extend(["", "## Native View"])
        for key, value in native_view.items():
            if isinstance(value, list):
                summary_lines.append(f"- `{key}`: {len(value)} item(s)")
                for item in value[:6]:
                    if isinstance(item, dict):
                        name = item.get("name") or item.get("repo") or item.get("path") or "item"
                        count = item.get("count")
                        extra = f" ({count})" if count is not None else ""
                        summary_lines.append(f"  - `{name}`{extra}")
                    else:
                        summary_lines.append(f"  - `{item}`")
            elif isinstance(value, dict):
                summary_lines.append(f"- `{key}`: {json.dumps(value, sort_keys=True)}")
            else:
                summary_lines.append(f"- `{key}`: {value}")
    summary_path.write_text("\n".join(summary_lines) + "\n", encoding="utf-8")
    return {
        "adapter": payload["adapter"],
        "entry_count": payload["entry_count"],
        "entry_type_counts": payload["entry_type_counts"],
        "entrypoints": payload["entrypoints"],
        "groups": payload["groups"],
        "license": payload["license"],
        "readme": payload["readme"],
        "repo_candidates": payload["repo_candidates"],
        "native_view": payload["native_view"],
        "publisher": payload["publisher"],
        "trust_level": payload["trust_level"],
        "trust_score": payload["trust_score"],
        "supported_agents": payload["supported_agents"],
        "repository_url": payload["repository_url"],
        "docs_url": payload["docs_url"],
        "index_path": str(index_path),
        "summary_path": str(summary_path),
    }


def _policy_allows_source(project_root: str | Path, source: ExternalSkillSource) -> tuple[bool, str | None]:
    config = load_config(Path(project_root).resolve())
    allowlist = {slug for slug in config.external_skills_allowlist}
    denylist = {slug for slug in config.external_skills_denylist}
    allowed_trust = {trust for trust in config.external_skills_allowed_trust_levels}
    policy_mode = config.external_skills_policy_mode
    if source.slug in denylist:
        return False, "source is denylisted by config"
    if allowlist and source.slug not in allowlist:
        return False, "source is not allowlisted by config"
    if allowed_trust and source.trust_level not in allowed_trust:
        return False, f"trust level '{source.trust_level}' is blocked by config"
    if policy_mode == "official_only" and source.trust_level not in {"official", "spec"}:
        return False, "policy mode official_only only permits official/spec sources"
    return True, None


def _upsert_lock_entry(project_root: str | Path, entry: dict[str, object]) -> None:
    lock = _load_lock(project_root)
    skills = [item for item in lock.get("skills", []) if isinstance(item, dict) and item.get("slug") != entry.get("slug")]
    skills.append(entry)
    lock["skills"] = sorted(skills, key=lambda item: str(item.get("slug", "")))
    _write_lock(project_root, lock)


def list_external_skills(
    project_root: str | Path = ".",
    *,
    ecosystem: str | None = None,
    search: str | None = None,
) -> dict[str, object]:
    installed = _installed_by_slug(project_root)
    locked = _lock_by_slug(project_root)
    items: list[dict[str, object]] = []
    needle = search.lower() if search else None
    for entry in CATALOG:
        if ecosystem and entry.ecosystem != ecosystem:
            continue
        haystack = " ".join(
            (
                entry.slug,
                entry.name,
                entry.ecosystem,
                entry.publisher,
                entry.description,
                " ".join(entry.tags),
                " ".join(entry.supported_agents),
            )
        ).lower()
        if needle and needle not in haystack:
            continue
        payload = _serialize_entry(entry, installed.get(entry.slug))
        payload["lock_metadata"] = locked.get(entry.slug)
        payload["active"] = bool(locked.get(entry.slug, {}).get("active", False))
        items.append(payload)
    return {
        "skills": items,
        "ecosystems": sorted({entry["ecosystem"] for entry in items}),
        "count": len(items),
    }


def ranked_external_skills(project_root: str | Path = ".") -> dict[str, object]:
    items = prioritized_active_external_skills(project_root)
    return {
        "skills": items,
        "count": len(items),
    }


def external_skill_policy(project_root: str | Path = ".") -> dict[str, object]:
    config = load_config(Path(project_root).resolve())
    return {
        "policy_mode": config.external_skills_policy_mode,
        "auto_install_enabled": config.auto_install_external_skills,
        "auto_activate_enabled": config.external_skills_auto_activate,
        "allowed_trust_levels": list(config.external_skills_allowed_trust_levels),
        "allowlist": list(config.external_skills_allowlist),
        "denylist": list(config.external_skills_denylist),
        "review_required_for_auto_activation": config.external_skills_policy_mode == "review_required",
    }


def get_external_skill(slug: str, project_root: str | Path = ".") -> dict[str, object]:
    entry = _catalog_entry(slug)
    if entry is None:
        raise KeyError(f"Unknown external skill source: {slug}")
    installed = _installed_by_slug(project_root).get(slug)
    payload = _serialize_entry(entry, installed)
    payload["lock_metadata"] = _lock_by_slug(project_root).get(slug)
    payload["active"] = bool(payload["lock_metadata"].get("active", False)) if isinstance(payload["lock_metadata"], dict) else False
    return payload


def _build_install_metadata(
    *,
    slug: str,
    name: str,
    ecosystem: str,
    repository_url: str,
    source_path: str | None,
    docs_url: str,
    trust_level: str,
    description: str,
    install_path: Path,
    install_mode: str = "manual",
    detection_reasons: list[str] | None = None,
    imported_from: str | None = None,
) -> dict[str, object]:
    source = _catalog_entry(slug)
    return {
        "slug": slug,
        "name": name,
        "ecosystem": ecosystem,
        "publisher": source.publisher if source is not None else "Custom",
        "category": source.category if source is not None else "custom",
        "supported_agents": list(source.supported_agents) if source is not None else [],
        "repository_url": repository_url,
        "source_path": source_path,
        "docs_url": docs_url,
        "trust_level": trust_level,
        "trust_score": TRUST_SCORES.get(trust_level, 0),
        "description": description,
        "install_path": str(install_path),
        "installed_at": datetime.now(UTC).isoformat(),
        "install_mode": install_mode,
        "detection_reasons": detection_reasons or [],
        "imported_from": imported_from,
    }


def install_external_skill(
    *,
    project_root: str | Path = ".",
    slug: str | None = None,
    git_url: str | None = None,
    name: str | None = None,
    force: bool = False,
    install_mode: str = "manual",
    detection_reasons: list[str] | None = None,
    ref: str | None = None,
    active: bool | None = None,
    ecosystem_override: str | None = None,
    docs_url_override: str | None = None,
    trust_level_override: str | None = None,
    category_override: str | None = None,
    publisher_override: str | None = None,
    description_override: str | None = None,
    imported_from: str | None = None,
) -> dict[str, object]:
    if slug is None and git_url is None:
        raise ValueError("Provide either a catalog slug or a git_url.")

    if slug is not None:
        entry = _catalog_entry(slug)
        if entry is None:
            raise KeyError(f"Unknown external skill source: {slug}")
        allowed, reason = _policy_allows_source(project_root, entry)
        if not allowed:
            raise PermissionError(f"Cannot install {entry.slug}: {reason}")
        resolved_slug = entry.slug
        resolved_name = entry.name
        repository_url = entry.repository_url
        ecosystem = entry.ecosystem
        source_path = entry.source_path
        docs_url = entry.docs_url
        trust_level = entry.trust_level
        description = entry.description
    else:
        assert git_url is not None
        resolved_name = name or Path(git_url.rstrip("/")).stem or "external-skill"
        resolved_slug = _normalize_slug(resolved_name)
        repository_url = git_url
        ecosystem = "custom"
        source_path = None
        docs_url = git_url
        trust_level = "custom"
        description = f"Custom external skill source installed from {git_url}."
    if ecosystem_override:
        ecosystem = ecosystem_override
    if docs_url_override:
        docs_url = docs_url_override
    if trust_level_override:
        trust_level = trust_level_override
    if description_override:
        description = description_override

    sources_dir = _sources_dir(project_root)
    sources_dir.mkdir(parents=True, exist_ok=True)
    install_path = sources_dir / resolved_slug
    if install_path.exists():
        if not force:
            installed = _installed_by_slug(project_root).get(resolved_slug)
            if installed is not None:
                return installed
            raise FileExistsError(f"{install_path} already exists. Use force=True to reinstall.")
        shutil.rmtree(install_path)

    subprocess.run(
        ["git", "clone", "--depth", "1", repository_url, str(install_path)],
        check=True,
        capture_output=True,
        text=True,
    )

    metadata = _build_install_metadata(
        slug=resolved_slug,
        name=resolved_name,
        ecosystem=ecosystem,
        repository_url=repository_url,
        source_path=source_path,
        docs_url=docs_url,
        trust_level=trust_level,
        description=description,
        install_path=install_path,
        install_mode=install_mode,
        detection_reasons=detection_reasons,
        imported_from=imported_from,
    )
    metadata["requested_ref"] = ref
    (install_path / "skilgen-source.json").write_text(json.dumps(metadata, indent=2), encoding="utf-8")

    if ref:
        subprocess.run(
            ["git", "-C", str(install_path), "checkout", ref],
            check=True,
            capture_output=True,
            text=True,
        )

    resolved_revision = _git_revision(install_path)
    if resolved_revision is not None:
        metadata["resolved_revision"] = resolved_revision
    remote_url = _git_remote_url(install_path)
    if remote_url is not None:
        metadata["repository_url"] = remote_url
    license_info = _detect_license(install_path)
    if license_info is not None:
        metadata["license"] = license_info

    source = entry if slug is not None and entry is not None else ExternalSkillSource(
        slug=resolved_slug,
        name=resolved_name,
        ecosystem=ecosystem,
        publisher=publisher_override or "Custom",
        description=description,
        repository_url=repository_url,
        source_path=source_path,
        docs_url=docs_url,
        category=category_override or "custom",
        trust_level=trust_level,
    )
    normalization = _normalize_external_skill_install(project_root, source=source, install_path=install_path)
    config = load_config(Path(project_root).resolve())
    auto_activate_allowed = config.external_skills_auto_activate and config.external_skills_policy_mode != "review_required"
    active_value = auto_activate_allowed if active is None else active
    metadata["normalized"] = normalization
    metadata["active"] = active_value
    metadata["publisher"] = publisher_override or metadata.get("publisher") or source.publisher
    metadata["category"] = category_override or metadata.get("category") or source.category
    metadata["trust_score"] = normalization.get("trust_score", TRUST_SCORES.get(trust_level, 0))
    metadata["provenance"] = _build_provenance_payload(
        source=source,
        install_path=install_path,
        requested_ref=ref,
        resolved_revision=resolved_revision,
        remote_url=remote_url,
        license_info=license_info,
        readme_info=metadata.get("normalized", {}).get("readme") if isinstance(metadata.get("normalized"), dict) else None,
        install_mode=install_mode,
        imported_from=imported_from,
    )

    manifest = _load_manifest(project_root)
    skills = [entry for entry in manifest.get("skills", []) if isinstance(entry, dict) and entry.get("slug") != resolved_slug]
    skills.append(metadata)
    manifest["skills"] = sorted(skills, key=lambda entry: str(entry.get("slug", "")))
    _write_manifest(project_root, manifest)
    _upsert_lock_entry(
        project_root,
        {
            "slug": resolved_slug,
            "name": resolved_name,
            "publisher": metadata.get("publisher"),
            "category": metadata.get("category"),
            "repository_url": metadata["repository_url"],
            "requested_ref": ref,
            "resolved_revision": resolved_revision,
            "active": active_value,
            "normalized": normalization,
            "license": metadata.get("license"),
            "provenance": metadata.get("provenance"),
            "trust_level": trust_level,
            "trust_score": metadata.get("trust_score", TRUST_SCORES.get(trust_level, 0)),
            "supported_agents": metadata.get("supported_agents", []),
            "install_mode": install_mode,
            "imported_from": imported_from,
            "updated_at": datetime.now(UTC).isoformat(),
        },
    )
    (install_path / "skilgen-source.json").write_text(json.dumps(metadata, indent=2), encoding="utf-8")
    return metadata


def ensure_external_skills_for_project(project_root: str | Path = ".") -> dict[str, object]:
    root = Path(project_root).resolve()
    config = load_config(root)
    detections = detect_external_skill_sources(root)
    installed = _installed_by_slug(root)
    locked = _lock_by_slug(root)
    newly_installed: list[dict[str, object]] = []
    already_installed: list[dict[str, object]] = []
    errors: list[dict[str, str]] = []
    blocked: list[dict[str, str]] = []
    for detection in detections["detected_skills"]:
        slug = str(detection["slug"])
        source = _catalog_entry(slug)
        if source is None:
            continue
        allowed, reason = _policy_allows_source(root, source)
        if not allowed:
            blocked.append({"slug": slug, "reason": reason or "blocked"})
            continue
        if slug in installed:
            auto_activate_allowed = config.external_skills_auto_activate and config.external_skills_policy_mode != "review_required"
            if auto_activate_allowed:
                lock_entry = locked.get(slug)
                if isinstance(lock_entry, dict) and not lock_entry.get("active"):
                    lock_entry["active"] = True
                    lock_entry["updated_at"] = datetime.now(UTC).isoformat()
                    _upsert_lock_entry(root, lock_entry)
            already_installed.append(installed[slug])
            continue
        try:
            metadata = install_external_skill(
                project_root=root,
                slug=slug,
                install_mode="auto",
                detection_reasons=[str(reason) for reason in detection.get("reasons", [])],
                active=config.external_skills_auto_activate and config.external_skills_policy_mode != "review_required",
            )
            newly_installed.append(metadata)
            installed[slug] = metadata
        except Exception as exc:  # pragma: no cover
            errors.append({"slug": slug, "error": str(exc)})
    return {
        **detections,
        "installed_skills": installed_external_skills(root),
        "newly_installed": newly_installed,
        "already_installed": already_installed,
        "errors": errors,
        "blocked": blocked,
    }


def sync_external_skill(*, project_root: str | Path = ".", slug: str) -> dict[str, object]:
    installed = _installed_by_slug(project_root).get(slug)
    if installed is None:
        raise KeyError(f"External skill source is not installed: {slug}")
    install_path = Path(str(installed["install_path"]))
    result = subprocess.run(
        ["git", "-C", str(install_path), "pull", "--ff-only"],
        check=True,
        capture_output=True,
        text=True,
    )
    source = _catalog_entry(slug) or ExternalSkillSource(
        slug=slug,
        name=str(installed.get("name", slug)),
        ecosystem=str(installed.get("ecosystem", "custom")),
        publisher="Custom",
        description=str(installed.get("description", "")),
        repository_url=str(installed.get("repository_url", "")),
        source_path=installed.get("source_path") if isinstance(installed.get("source_path"), str) or installed.get("source_path") is None else None,
        docs_url=str(installed.get("docs_url", installed.get("repository_url", ""))),
        category="custom",
        trust_level=str(installed.get("trust_level", "custom")),
    )
    normalization = _normalize_external_skill_install(project_root, source=source, install_path=install_path)
    installed["synced_at"] = datetime.now(UTC).isoformat()
    installed["sync_stdout"] = result.stdout.strip()
    installed["sync_stderr"] = result.stderr.strip()
    installed["resolved_revision"] = _git_revision(install_path)
    installed["license"] = _detect_license(install_path)
    installed["normalized"] = normalization
    installed["trust_score"] = normalization.get("trust_score", installed.get("trust_score", 0))
    installed["provenance"] = _build_provenance_payload(
        source=source,
        install_path=install_path,
        requested_ref=installed.get("requested_ref") if isinstance(installed.get("requested_ref"), str) or installed.get("requested_ref") is None else None,
        resolved_revision=installed.get("resolved_revision") if isinstance(installed.get("resolved_revision"), str) or installed.get("resolved_revision") is None else None,
        remote_url=_git_remote_url(install_path),
        license_info=installed.get("license") if isinstance(installed.get("license"), dict) or installed.get("license") is None else None,
        readme_info=normalization.get("readme") if isinstance(normalization.get("readme"), dict) or normalization.get("readme") is None else None,
        install_mode=str(installed.get("install_mode", "manual")),
        imported_from=str(installed.get("imported_from")) if installed.get("imported_from") is not None else None,
    )
    manifest = _load_manifest(project_root)
    manifest["skills"] = sorted(
        [
            installed if isinstance(entry, dict) and entry.get("slug") == slug else entry
            for entry in manifest.get("skills", [])
            if isinstance(entry, dict)
        ],
        key=lambda entry: str(entry.get("slug", "")),
    )
    _write_manifest(project_root, manifest)
    (install_path / "skilgen-source.json").write_text(json.dumps(installed, indent=2), encoding="utf-8")
    lock_entry = _lock_by_slug(project_root).get(slug, {"slug": slug})
    lock_entry.update(
        {
            "slug": slug,
            "name": installed.get("name"),
            "publisher": installed.get("publisher"),
            "category": installed.get("category"),
            "repository_url": installed.get("repository_url"),
            "requested_ref": installed.get("requested_ref"),
            "resolved_revision": installed.get("resolved_revision"),
            "active": lock_entry.get("active", False),
            "normalized": normalization,
            "license": installed.get("license"),
            "provenance": installed.get("provenance"),
            "trust_level": installed.get("trust_level"),
            "trust_score": installed.get("trust_score", normalization.get("trust_score", TRUST_SCORES.get(str(installed.get("trust_level", "custom")), 0))),
            "supported_agents": installed.get("supported_agents", []),
            "install_mode": installed.get("install_mode", "manual"),
            "imported_from": installed.get("imported_from"),
            "updated_at": datetime.now(UTC).isoformat(),
        }
    )
    _upsert_lock_entry(project_root, lock_entry)
    return installed


def sync_all_external_skills(*, project_root: str | Path = ".") -> dict[str, object]:
    synced: list[dict[str, object]] = []
    errors: list[dict[str, str]] = []
    for installed in installed_external_skills(project_root):
        slug = str(installed["slug"])
        try:
            synced.append(sync_external_skill(project_root=project_root, slug=slug))
        except Exception as exc:  # pragma: no cover
            errors.append({"slug": slug, "error": str(exc)})
    return {"skills": synced, "errors": errors, "count": len(synced)}


def activate_external_skill(*, project_root: str | Path = ".", slug: str) -> dict[str, object]:
    installed = _installed_by_slug(project_root).get(slug)
    if installed is None:
        raise KeyError(f"External skill source is not installed: {slug}")
    lock_entry = _lock_by_slug(project_root).get(slug, {"slug": slug})
    lock_entry["active"] = True
    lock_entry["updated_at"] = datetime.now(UTC).isoformat()
    lock_entry.setdefault("publisher", installed.get("publisher"))
    lock_entry.setdefault("category", installed.get("category"))
    lock_entry.setdefault("license", installed.get("license"))
    lock_entry.setdefault("provenance", installed.get("provenance"))
    lock_entry.setdefault("trust_level", installed.get("trust_level"))
    lock_entry.setdefault("trust_score", installed.get("trust_score", TRUST_SCORES.get(str(installed.get("trust_level", "custom")), 0)))
    lock_entry.setdefault("supported_agents", installed.get("supported_agents", []))
    if "normalized" not in lock_entry:
        install_path = Path(str(installed["install_path"]))
        source = _catalog_entry(slug) or ExternalSkillSource(
            slug=slug,
            name=str(installed.get("name", slug)),
            ecosystem=str(installed.get("ecosystem", "custom")),
            publisher="Custom",
            description=str(installed.get("description", "")),
            repository_url=str(installed.get("repository_url", "")),
            source_path=installed.get("source_path") if isinstance(installed.get("source_path"), str) or installed.get("source_path") is None else None,
            docs_url=str(installed.get("docs_url", installed.get("repository_url", ""))),
            category="custom",
            trust_level=str(installed.get("trust_level", "custom")),
        )
        lock_entry["normalized"] = _normalize_external_skill_install(project_root, source=source, install_path=install_path)
    _upsert_lock_entry(project_root, lock_entry)
    return {"slug": slug, "active": True, "lock_metadata": lock_entry}


def deactivate_external_skill(*, project_root: str | Path = ".", slug: str) -> dict[str, object]:
    installed = _installed_by_slug(project_root).get(slug)
    if installed is None:
        raise KeyError(f"External skill source is not installed: {slug}")
    lock_entry = _lock_by_slug(project_root).get(slug, {"slug": slug})
    lock_entry["active"] = False
    lock_entry["updated_at"] = datetime.now(UTC).isoformat()
    lock_entry.setdefault("publisher", installed.get("publisher"))
    lock_entry.setdefault("category", installed.get("category"))
    lock_entry.setdefault("license", installed.get("license"))
    lock_entry.setdefault("provenance", installed.get("provenance"))
    lock_entry.setdefault("trust_level", installed.get("trust_level"))
    lock_entry.setdefault("trust_score", installed.get("trust_score", TRUST_SCORES.get(str(installed.get("trust_level", "custom")), 0)))
    lock_entry.setdefault("supported_agents", installed.get("supported_agents", []))
    _upsert_lock_entry(project_root, lock_entry)
    return {"slug": slug, "active": False, "lock_metadata": lock_entry}


def remove_external_skill(*, project_root: str | Path = ".", slug: str) -> dict[str, object]:
    installed = _installed_by_slug(project_root).get(slug)
    if installed is None:
        raise KeyError(f"External skill source is not installed: {slug}")
    install_path = Path(str(installed["install_path"]))
    if install_path.exists():
        shutil.rmtree(install_path)
    normalized_path = _normalized_dir(project_root) / slug
    if normalized_path.exists():
        shutil.rmtree(normalized_path)
    manifest = _load_manifest(project_root)
    manifest["skills"] = [
        entry
        for entry in manifest.get("skills", [])
        if isinstance(entry, dict) and entry.get("slug") != slug
    ]
    _write_manifest(project_root, manifest)
    lock = _load_lock(project_root)
    lock["skills"] = [
        entry
        for entry in lock.get("skills", [])
        if isinstance(entry, dict) and entry.get("slug") != slug
    ]
    _write_lock(project_root, lock)
    return {
        "slug": slug,
        "removed": True,
        "install_path": str(install_path),
    }


def import_external_skill_candidates(
    *,
    project_root: str | Path = ".",
    slug: str,
    limit: int = 5,
    active: bool | None = None,
) -> dict[str, object]:
    installed = _installed_by_slug(project_root).get(slug)
    if installed is None:
        raise KeyError(f"External skill source is not installed: {slug}")
    normalized = installed.get("normalized", {}) if isinstance(installed.get("normalized"), dict) else {}
    repo_candidates = normalized.get("repo_candidates", []) if isinstance(normalized, dict) else []
    if not isinstance(repo_candidates, list):
        repo_candidates = []
    imported: list[dict[str, object]] = []
    skipped: list[dict[str, str]] = []
    errors: list[dict[str, str]] = []
    parent_trust = str(installed.get("trust_level", "directory"))
    inherited_trust = "curated" if parent_trust == "curated" else "community"
    for candidate in repo_candidates[: max(limit, 0)]:
        if not isinstance(candidate, dict):
            continue
        repo = str(candidate.get("repo", "")).strip()
        url = str(candidate.get("url", "")).strip()
        if not repo or not url:
            continue
        candidate_name = repo.replace("/", "-")
        candidate_slug = _normalize_slug(candidate_name)
        if candidate_slug in _installed_by_slug(project_root):
            skipped.append({"slug": candidate_slug, "reason": "already_installed"})
            continue
        try:
            imported.append(
                install_external_skill(
                    project_root=project_root,
                    git_url=url,
                    name=candidate_name,
                    install_mode="directory_import",
                    active=active,
                    ecosystem_override="imported",
                    trust_level_override=inherited_trust,
                    category_override="directory-import",
                    publisher_override=repo.split("/", 1)[0],
                    description_override=f"Imported from directory source '{slug}' via {repo}.",
                    imported_from=slug,
                )
            )
        except Exception as exc:  # pragma: no cover
            errors.append({"repo": repo, "error": str(exc)})
    return {
        "source_slug": slug,
        "imported_skills": imported,
        "skipped": skipped,
        "errors": errors,
        "count": len(imported),
    }


def export_external_skill_lock(*, project_root: str | Path = ".", output_path: str | Path | None = None) -> dict[str, object]:
    root = Path(project_root).resolve()
    lock = _load_lock(root)
    destination = Path(output_path).resolve() if output_path is not None else _export_lock_path(root)
    destination.parent.mkdir(parents=True, exist_ok=True)
    payload = {
        "exported_at": datetime.now(UTC).isoformat(),
        "project_root": str(root),
        "skills": lock.get("skills", []),
    }
    destination.write_text(json.dumps(payload, indent=2), encoding="utf-8")
    return {
        "export_path": str(destination),
        "count": len(payload["skills"]) if isinstance(payload["skills"], list) else 0,
    }


def import_external_skill_lock(
    *,
    project_root: str | Path = ".",
    input_path: str | Path,
    sync_existing: bool = False,
) -> dict[str, object]:
    root = Path(project_root).resolve()
    payload = json.loads(Path(input_path).resolve().read_text(encoding="utf-8"))
    entries = payload.get("skills", [])
    if not isinstance(entries, list):
        entries = []
    imported: list[dict[str, object]] = []
    updated: list[dict[str, object]] = []
    skipped: list[dict[str, str]] = []
    errors: list[dict[str, str]] = []
    installed = _installed_by_slug(root)
    for entry in entries:
        if not isinstance(entry, dict):
            continue
        slug = str(entry.get("slug", "")).strip()
        repo_url = str(entry.get("repository_url", "")).strip()
        requested_ref = entry.get("requested_ref") if isinstance(entry.get("requested_ref"), str) or entry.get("requested_ref") is None else None
        if not slug or not repo_url:
            skipped.append({"slug": slug or "unknown", "reason": "missing_repository_url"})
            continue
        if slug in installed:
            if sync_existing:
                try:
                    synced = sync_external_skill(project_root=root, slug=slug)
                    desired_active = bool(entry.get("active", False))
                    if desired_active:
                        activate_external_skill(project_root=root, slug=slug)
                    else:
                        deactivate_external_skill(project_root=root, slug=slug)
                    updated.append(synced)
                except Exception as exc:  # pragma: no cover
                    errors.append({"slug": slug, "error": str(exc)})
            else:
                skipped.append({"slug": slug, "reason": "already_installed"})
            continue
        try:
            imported.append(
                install_external_skill(
                    project_root=root,
                    slug=slug if _catalog_entry(slug) is not None else None,
                    git_url=None if _catalog_entry(slug) is not None else repo_url,
                    name=None if _catalog_entry(slug) is not None else str(entry.get("name", slug)),
                    ref=requested_ref,
                    active=bool(entry.get("active", False)),
                    install_mode="lock_import",
                    ecosystem_override=None if _catalog_entry(slug) is not None else str(entry.get("ecosystem", "imported")),
                    trust_level_override=None if _catalog_entry(slug) is not None else str(entry.get("trust_level", "custom")),
                    category_override=None if _catalog_entry(slug) is not None else str(entry.get("category", "lock-import")),
                    publisher_override=None if _catalog_entry(slug) is not None else str(entry.get("publisher", "Imported")),
                    docs_url_override=None if _catalog_entry(slug) is not None else str(entry.get("provenance", {}).get("docs_url", repo_url)) if isinstance(entry.get("provenance"), dict) else repo_url,
                    description_override=None if _catalog_entry(slug) is not None else f"Imported from external skills lock for {slug}.",
                    imported_from=str(entry.get("imported_from")) if entry.get("imported_from") is not None else None,
                )
            )
        except Exception as exc:  # pragma: no cover
            errors.append({"slug": slug, "error": str(exc)})
    return {
        "imported_skills": imported,
        "updated_skills": updated,
        "skipped": skipped,
        "errors": errors,
        "count": len(imported) + len(updated),
    }



from __future__ import annotations

from pathlib import Path

from skilgen.agents.codebase_signals import analyze_codebase
from skilgen.deep_agents_core import run_deep_json
from skilgen.core.models import ProjectIntent
from skilgen.core.requirements import extract_project_intent, extract_text, normalize_lines


def parse_requirements_file_native(path: Path) -> ProjectIntent:
    return extract_project_intent(normalize_lines(extract_text(path)))


def parse_requirements_file(path: Path) -> ProjectIntent:
    resolved = path.resolve()
    text = extract_text(resolved)
    native_intent = extract_project_intent(normalize_lines(text))
    payload = run_deep_json(
        "requirements interpretation",
        (
            "Interpret the following requirements for Skilgen's planning layer. Return JSON with keys "
            "features, domain_concepts, entities, endpoints, ui_flows. Each key must contain a short list of "
            "implementation-relevant strings. Prefer concrete nouns, capabilities, routes, workflows, and "
            "business concepts over vague summaries. Extract both explicit requirements and strongly implied "
            "delivery expectations, but do not invent unsupported details. If the requirements mention a flow, "
            "translate it into a reusable UI or product flow label. If the requirements imply API or backend "
            "behavior, extract endpoint-oriented or service-oriented phrases that a coding agent could act on.\n\n"
            f"Requirements file: {resolved}\n"
            f"Content:\n{text}"
        ),
        lambda: {
            "features": native_intent.features,
            "domain_concepts": native_intent.domain_concepts,
            "entities": native_intent.entities,
            "endpoints": native_intent.endpoints,
            "ui_flows": native_intent.ui_flows,
        },
        project_root=resolved.parent,
    )
    return ProjectIntent(
        features=[str(item) for item in payload.get("features", [])][:12],
        domain_concepts=[str(item) for item in payload.get("domain_concepts", [])][:12],
        entities=[str(item) for item in payload.get("entities", [])][:12],
        endpoints=[str(item) for item in payload.get("endpoints", [])][:12],
        ui_flows=[str(item) for item in payload.get("ui_flows", [])][:12],
    )


def parse_project_intent_native(project_root: Path, requirements_path: Path | None = None) -> ProjectIntent:
    if requirements_path is not None:
        return parse_requirements_file_native(requirements_path.resolve())
    signals = analyze_codebase(project_root.resolve())
    features = ["Codebase-only scan", "Generate skills from the current repository structure"]
    domain_concepts: list[str] = []
    entities: list[str] = []
    endpoints = [f"Detected route: {item}" for item in signals.backend_routes[:12]]
    ui_flows = [f"Detected route: {item}" for item in signals.frontend_routes[:12]]
    if signals.backend_routes or signals.services:
        domain_concepts.append("backend")
    if signals.frontend_routes or signals.components:
        domain_concepts.append("frontend")
    entities.extend(f"Service: {item}" for item in signals.services[:6])
    entities.extend(f"Component: {item}" for item in signals.components[:6])
    features.extend(f"Backend route: {item}" for item in signals.backend_routes[:6])
    features.extend(f"Frontend route: {item}" for item in signals.frontend_routes[:6])
    return ProjectIntent(
        features=features[:12],
        domain_concepts=domain_concepts[:12],
        entities=entities[:12],
        endpoints=endpoints[:12],
        ui_flows=ui_flows[:12],
    )


def parse_project_intent(project_root: Path, requirements_path: Path | None = None) -> ProjectIntent:
    if requirements_path is not None:
        return parse_requirements_file(requirements_path.resolve())
    resolved_root = project_root.resolve()
    native_intent = parse_project_intent_native(resolved_root, None)
    signals = analyze_codebase(resolved_root)
    payload = run_deep_json(
        "codebase intent interpretation",
        (
            "Interpret the repository structure for Skilgen's intent layer and return JSON with keys "
            "features, domain_concepts, entities, endpoints, ui_flows. Each key must contain a short list of "
            "strings grounded in the detected codebase. Infer likely product and engineering concerns from the "
            "actual structure: routes, components, services, tests, auth, data models, and design-system files. "
            "Prefer names that would help a coding agent choose the right skills and plan the next implementation step.\n\n"
            f"Project root: {resolved_root}\n"
            f"Backend routes: {signals.backend_routes}\n"
            f"Frontend routes: {signals.frontend_routes}\n"
            f"Components: {signals.components}\n"
            f"Services: {signals.services}\n"
            f"Tests: {signals.tests}\n"
        ),
        lambda: {
            "features": native_intent.features,
            "domain_concepts": native_intent.domain_concepts,
            "entities": native_intent.entities,
            "endpoints": native_intent.endpoints,
            "ui_flows": native_intent.ui_flows,
        },
        project_root=resolved_root,
    )
    return ProjectIntent(
        features=[str(item) for item in payload.get("features", [])][:12],
        domain_concepts=[str(item) for item in payload.get("domain_concepts", [])][:12],
        entities=[str(item) for item in payload.get("entities", [])][:12],
        endpoints=[str(item) for item in payload.get("endpoints", [])][:12],
        ui_flows=[str(item) for item in payload.get("ui_flows", [])][:12],
    )



from __future__ import annotations

import hashlib
import json
from dataclasses import asdict
from pathlib import Path

from skilgen.core.models import DomainGraph, FreshnessReport, FreshnessState, RequirementsContext


IGNORED_PARTS = {
    ".git",
    ".venv",
    "venv",
    "node_modules",
    "__pycache__",
    "dist",
    "build",
}
IGNORED_FILES = {
    "AGENTS.md",
    "ANALYSIS.md",
    "FEATURES.md",
    "REPORT.md",
    "TRACEABILITY.md",
}


def _state_dir(project_root: Path) -> Path:
    return project_root.resolve() / ".skilgen" / "state"


def _state_path(project_root: Path) -> Path:
    return _state_dir(project_root) / "freshness.json"


def _iter_source_files(project_root: Path) -> list[Path]:
    root = project_root.resolve()
    files: list[Path] = []
    for path in root.rglob("*"):
        if not path.is_file():
            continue
        relative = path.relative_to(root)
        if set(relative.parts) & IGNORED_PARTS:
            continue
        if relative.parts and relative.parts[0] == "skills":
            continue
        if relative.parts and relative.parts[0] == ".skilgen":
            continue
        if path.name in IGNORED_FILES:
            continue
        files.append(path)
    return sorted(files)


def _hash_file(path: Path) -> str:
    return hashlib.sha256(path.read_bytes()).hexdigest()


def _top_level_domains(domain_graph: DomainGraph) -> list[str]:
    return [node.name for node in domain_graph.nodes if node.parent_domain is None and node.skill_path]


def snapshot_freshness_state(
    project_root: Path,
    requirements: RequirementsContext,
    domain_graph: DomainGraph,
) -> FreshnessState:
    root = project_root.resolve()
    source_hashes = {path.relative_to(root).as_posix(): _hash_file(path) for path in _iter_source_files(root)}
    return FreshnessState(
        source_hashes=source_hashes,
        requirements_source_hash=requirements.source_hash,
        domain_graph_nodes=[asdict(node) for node in domain_graph.nodes],
        top_level_domains=_top_level_domains(domain_graph),
    )


def load_freshness_state(project_root: Path) -> FreshnessState | None:
    path = _state_path(project_root)
    if not path.exists():
        return None
    payload = json.loads(path.read_text(encoding="utf-8"))
    return FreshnessState(
        source_hashes={str(key): str(value) for key, value in payload.get("source_hashes", {}).items()},
        requirements_source_hash=str(payload.get("requirements_source_hash", "")),
        domain_graph_nodes=list(payload.get("domain_graph_nodes", [])),
        top_level_domains=[str(item) for item in payload.get("top_level_domains", [])],
    )


def save_freshness_state(project_root: Path, state: FreshnessState) -> Path:
    path = _state_path(project_root)
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(json.dumps(asdict(state), indent=2, sort_keys=True), encoding="utf-8")
    return path


def compute_freshness_report(
    project_root: Path,
    requirements: RequirementsContext,
    domain_graph: DomainGraph,
    previous_state: FreshnessState | None,
) -> FreshnessReport:
    current_state = snapshot_freshness_state(project_root, requirements, domain_graph)
    top_level_domains = current_state.top_level_domains
    if previous_state is None:
        stale_paths = [node.skill_path for node in domain_graph.nodes if node.skill_path]
        return FreshnessReport(
            changed_files=sorted(current_state.source_hashes),
            impacted_domains=top_level_domains,
            stale_skill_paths=sorted(path for path in stale_paths if path),
            top_level_domains=top_level_domains,
            reason="initial_generation",
        )

    changed_files = sorted(
        path
        for path in set(previous_state.source_hashes) | set(current_state.source_hashes)
        if previous_state.source_hashes.get(path) != current_state.source_hashes.get(path)
    )

    if previous_state.requirements_source_hash != current_state.requirements_source_hash:
        changed_files.append("requirements")
    changed_files = sorted(set(changed_files))

    if not changed_files:
        return FreshnessReport(
            changed_files=[],
            impacted_domains=[],
            stale_skill_paths=[],
            top_level_domains=top_level_domains,
            reason="no_source_changes",
        )

    nodes_by_name = {node.name: node for node in domain_graph.nodes}
    impacted: set[str] = set()
    stale_paths: set[str] = set()

    if "requirements" in changed_files:
        stale_paths.update(node.skill_path for node in domain_graph.nodes if node.skill_path)
        return FreshnessReport(
            changed_files=changed_files,
            impacted_domains=top_level_domains,
            stale_skill_paths=sorted(path for path in stale_paths if path),
            top_level_domains=top_level_domains,
            reason="requirements_changed",
        )

    for node in domain_graph.nodes:
        if not node.skill_path:
            continue
        evidence = set(node.key_files)
        if evidence and any(changed in evidence for changed in changed_files):
            stale_paths.add(node.skill_path)
            impacted.add(node.parent_domain or node.name)
            parent_name = node.parent_domain
            while parent_name:
                impacted.add(parent_name)
                parent = nodes_by_name.get(parent_name)
                if parent and parent.skill_path:
                    stale_paths.add(parent.skill_path)
                parent_name = parent.parent_domain if parent else None

    top_level_set = set(top_level_domains)
    impacted = {domain for domain in impacted if domain in top_level_set}

    if not impacted:
        impacted = top_level_set
        stale_paths.update(node.skill_path for node in domain_graph.nodes if node.skill_path)

    reason = "requirements_changed" if "requirements" in changed_files else "source_changes_detected"
    return FreshnessReport(
        changed_files=changed_files,
        impacted_domains=sorted(impacted),
            stale_skill_paths=sorted(path for path in stale_paths if path),
            top_level_domains=top_level_domains,
            reason=reason,
        )



from __future__ import annotations

from pathlib import Path

from skilgen.agents.codebase_signals import analyze_codebase
from skilgen.deep_agents_core import runtime_diagnostics


def _parse_references(skill: Path) -> list[str]:
    references: list[str] = []
    lines = skill.read_text(encoding="utf-8").splitlines()
    in_refs = False
    for line in lines:
        stripped = line.strip()
        if stripped == "references:":
            in_refs = True
            continue
        if in_refs and stripped.startswith("- "):
            references.append(stripped[2:].strip())
        elif in_refs and stripped and not stripped.startswith("- "):
            in_refs = False
    return references


def _needs_bidirectional_check(root: Path, source: Path, target: Path) -> bool:
    try:
        source_parts = source.relative_to(root).parts
        target_parts = target.relative_to(root).parts
    except ValueError:
        return False
    if len(source_parts) < 3 or len(target_parts) < 3:
        return False
    if source_parts[0] != "skills" or target_parts[0] != "skills":
        return False
    source_domain = source_parts[1]
    target_domain = target_parts[1]
    if source_domain != target_domain:
        return False
    return True


def validate_project(project_root: str | Path) -> dict[str, object]:
    root = Path(project_root).resolve()
    errors: list[str] = []
    warnings: list[str] = []
    recommendations: list[str] = []
    manifest = root / "skills" / "MANIFEST.md"
    agents_contract = root / "AGENTS.md"
    if not manifest.exists():
        errors.append("skills/MANIFEST.md is missing")
    if not (root / "skills" / "GRAPH.md").exists():
        errors.append("skills/GRAPH.md is missing")
    if not (root / "TRACEABILITY.md").exists():
        warnings.append("TRACEABILITY.md is missing")
    if not agents_contract.exists():
        errors.append("AGENTS.md is missing")

    resolved_refs: dict[Path, set[Path]] = {}
    if (root / "skills").exists():
        for skill in (root / "skills").rglob("SKILL.md"):
            refs: set[Path] = set()
            for reference in _parse_references(skill):
                target = (skill.parent / reference).resolve()
                refs.add(target)
                if not target.exists():
                    errors.append(f"Missing reference from {skill.relative_to(root)} to {reference}")
            resolved_refs[skill.resolve()] = refs

    for skill, targets in resolved_refs.items():
        for target in targets:
            if target.name != "SKILL.md" or target not in resolved_refs:
                continue
            if not _needs_bidirectional_check(root, skill, target):
                continue
            if skill not in resolved_refs[target]:
                errors.append(
                    f"Missing bidirectional reference between {skill.relative_to(root)} and {target.relative_to(root)}"
                )

    signals = analyze_codebase(root)
    diagnostics = runtime_diagnostics(root)
    if diagnostics["runtime"] != "model_backed":
        warnings.append(f"Model-backed runtime is not ready: {diagnostics['reason']}")
        recommendations.extend(diagnostics["recommendations"])
    if signals.backend_routes and not (root / "skills" / "backend" / "routes" / "SKILL.md").exists():
        warnings.append("Backend route files were detected but skills/backend/routes/SKILL.md is missing")
    if signals.services and not (root / "skills" / "backend" / "services" / "SKILL.md").exists():
        warnings.append("Service files were detected but skills/backend/services/SKILL.md is missing")
    if signals.backend_routes and not signals.tests:
        warnings.append("Backend route files were detected but no test files were found")
    if signals.frontend_routes and not (root / "skills" / "frontend" / "routes" / "SKILL.md").exists():
        warnings.append("Frontend route files were detected but skills/frontend/routes/SKILL.md is missing")
    if signals.data_models and not (root / "TRACEABILITY.md").exists():
        warnings.append("Data model files were detected but TRACEABILITY.md is missing")
    if signals.persistence_layers and not (root / "skills" / "backend" / "services" / "SKILL.md").exists():
        warnings.append("Persistence files were detected but backend service guidance is missing")
    if signals.auth_files and not (root / "skills" / "backend" / "SKILL.md").exists():
        warnings.append("Auth files were detected but backend guidance is missing")
    if signals.state_files and not (root / "skills" / "frontend" / "SKILL.md").exists():
        warnings.append("State files were detected but frontend guidance is missing")
    if signals.design_system_files and not (root / "skills" / "frontend" / "components" / "SKILL.md").exists():
        warnings.append("Design system files were detected but frontend component guidance is missing")
    if agents_contract.exists():
        contract_text = agents_contract.read_text(encoding="utf-8")
        required_refs = [
            "skills/MANIFEST.md",
            "skills/requirements/SKILL.md",
            "skills/roadmap/SKILL.md",
        ]
        if signals.backend_routes or signals.services or signals.auth_files:
            required_refs.append("skills/backend/SKILL.md")
        if signals.frontend_routes or signals.components or signals.state_files:
            required_refs.append("skills/frontend/SKILL.md")
        for required_ref in required_refs:
            if required_ref not in contract_text:
                warnings.append(f"AGENTS.md does not reference {required_ref}")

    coverage = {
        "backend_routes": len(signals.backend_routes),
        "frontend_routes": len(signals.frontend_routes),
        "components": len(signals.components),
        "services": len(signals.services),
        "tests": len(signals.tests),
        "data_models": len(signals.data_models),
        "persistence_layers": len(signals.persistence_layers),
        "background_jobs": len(signals.background_jobs),
        "auth_files": len(signals.auth_files),
        "state_files": len(signals.state_files),
        "design_system_files": len(signals.design_system_files),
    }
    expected_checks = [
        (bool(signals.backend_routes), (root / "skills" / "backend" / "routes" / "SKILL.md").exists()),
        (bool(signals.services), (root / "skills" / "backend" / "services" / "SKILL.md").exists()),
        (bool(signals.frontend_routes), (root / "skills" / "frontend" / "routes" / "SKILL.md").exists()),
        (bool(signals.components), (root / "skills" / "frontend" / "components" / "SKILL.md").exists()),
        (True, (root / "TRACEABILITY.md").exists()),
        (True, manifest.exists()),
        (True, (root / "skills" / "GRAPH.md").exists()),
    ]
    applicable = [matched for required, matched in expected_checks if required]
    satisfied = sum(1 for matched in applicable if matched)
    base_score = int((satisfied / len(applicable)) * 100) if applicable else 100
    completeness_score = max(0, base_score - (len(warnings) * 5) - (len(errors) * 20))
    if signals.backend_routes and signals.tests:
        completeness_score = min(100, completeness_score + 5)
    if agents_contract.exists():
        completeness_score = min(100, completeness_score + 5)
    if not recommendations:
        recommendations.append("Runtime, agent contract, and generated skill coverage look healthy.")

    return {
        "valid": not errors,
        "errors": errors,
        "warnings": warnings,
        "coverage": coverage,
        "completeness_score": completeness_score,
        "runtime_diagnostics": diagnostics,
        "recommendations": recommendations,
    }



from pathlib import Path
from tempfile import TemporaryDirectory
import unittest

from skilgen.core.context import build_codebase_context
from skilgen.core.requirements import load_requirements


class ContextTests(unittest.TestCase):
    def test_build_codebase_context_includes_domains_and_skill_tree(self) -> None:
        with TemporaryDirectory() as tmp:
            root = Path(tmp)
            requirements = root / "requirements.md"
            requirements.write_text("Backend API endpoints\nFrontend routes\n", encoding="utf-8")
            (root / "api" / "routes").mkdir(parents=True)
            (root / "api" / "routes" / "scan.py").write_text("def handler():\n    return {}\n", encoding="utf-8")
            (root / "services").mkdir(parents=True)
            (root / "services" / "scan_service.py").write_text("def run():\n    return True\n", encoding="utf-8")
            (root / "models").mkdir(parents=True)
            (root / "models" / "scan_model.py").write_text("class Scan: pass\n", encoding="utf-8")
            (root / "src" / "routes").mkdir(parents=True)
            (root / "src" / "routes" / "dashboard.tsx").write_text("export default function Dashboard() { return null; }\n", encoding="utf-8")
            (root / "src" / "components").mkdir(parents=True)
            (root / "src" / "components" / "SkillCard.tsx").write_text("export function SkillCard() { return null; }\n", encoding="utf-8")
            (root / "src" / "theme").mkdir(parents=True)
            (root / "src" / "theme" / "tokens.ts").write_text("export const tokens = {};\n", encoding="utf-8")
            context = build_codebase_context(root, load_requirements(requirements))
            domain_names = {record.name for record in context.detected_domains}
            self.assertIn("requirements", domain_names)
            self.assertIn("backend", domain_names)
            self.assertIn("frontend", domain_names)
            self.assertIn("roadmap", domain_names)
            graph_names = {node.name for node in context.domain_graph.nodes}
            self.assertIn("backend-data", graph_names)
            self.assertIn("frontend-design-system", graph_names)
            self.assertTrue(context.skill_tree)
            backend_node = next(node for node in context.skill_tree if node.domain == "backend")
            frontend_node = next(node for node in context.skill_tree if node.domain == "frontend")
            self.assertIn("skills/backend/routes/SKILL.md", backend_node.child_skills)
            self.assertIn("skills/backend/services/SKILL.md", backend_node.child_skills)
            self.assertIn("skills/backend/data/SKILL.md", backend_node.child_skills)
            self.assertIn("skills/frontend/routes/SKILL.md", frontend_node.child_skills)
            self.assertIn("skills/frontend/design-system/SKILL.md", frontend_node.child_skills)

    def test_build_codebase_context_can_infer_freeform_top_level_domains(self) -> None:
        with TemporaryDirectory() as tmp:
            root = Path(tmp)
            requirements = root / "requirements.md"
            requirements.write_text("Shared tokens and design language\n", encoding="utf-8")
            (root / "src" / "theme").mkdir(parents=True)
            (root / "src" / "theme" / "tokens.ts").write_text("export const tokens = {};\n", encoding="utf-8")

            context = build_codebase_context(root, load_requirements(requirements))

            domain_names = {record.name for record in context.detected_domains}
            self.assertIn("design-system", domain_names)
            design_node = next(node for node in context.skill_tree if node.domain == "design-system")
            self.assertEqual(design_node.path, "skills/design-system/SKILL.md")


if __name__ == "__main__":
    unittest.main()
